package com.hexin.android.component.fenshitab.view

import android.content.Context
import android.text.TextUtils
import android.util.AttributeSet
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.TextView
import com.hexin.android.component.StockMarketUtils
import com.hexin.android.service.CBASConstants
import com.hexin.android.service.UnifyUrlManager
import com.hexin.android.theme.ThemeManager
import com.hexin.android.view.TagTextViewHelper
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.plat.android.R
import com.hexin.util.Log
import com.hexin.util.config.EQConstants
import java.lang.StringBuilder
import kotlin.experimental.or

class StockTagView(context: Context?, attrs: AttributeSet?) : LinearLayout(context, attrs),
        View.OnClickListener {

    companion object {
        private const val LOG_TAG = "PopStockTag"

        private const val TAG_N = "N"
        private const val TAG_XR = "XR"
        private const val TAG_XD = "XD"
        private const val TAG_DR = "DR"
        private const val YUAN = "元"
        private val CBAS_OBJ = arrayOf("rong", "zhiya", "ggt", "hsgt")
        private val CBAS_RES_ID = arrayOf(null, "free_guquanzhiya_stock_", "free_ggts_stock_", "free_hsgts_stock_")
        private const val RZRQ_STR = "融资余额%s，融券余额%s"
        private const val GZY_STR = "股权质押比例%s%%" // 如果要通过format显示--%，真正的%一定要双写转义
        private const val T_STR = "持股比例%s%%，净买入%s"

        fun getShowTagsCbas(flagStr: String): String {
            val flag: Int
            try {
                flag = Integer.valueOf(flagStr.trim())
            } catch (e: Exception) {
                Log.printStackTrace(e)
                return CBASConstants.CBAS_IPO_MOREN
            }

            val tags = StringBuilder()

            if ((flag and EQConstants.FLAG_RZRQ) == EQConstants.FLAG_RZRQ) { //融资融券标志位
                tags.append(CBAS_OBJ[0])
            }
            if ((flag and EQConstants.FLAG_GZY) == EQConstants.FLAG_GZY) { // 高质押
                if (tags.isNotEmpty()) {
                    tags.append(EQConstants.SIGN_PLUS)
                }
                tags.append(CBAS_OBJ[1])
            }
            if ((flag and EQConstants.FLAG_HUSHEN) == EQConstants.FLAG_HUSHEN ||
                    (flag and EQConstants.FLAG_HUSHEN) == EQConstants.FLAG_HUSHI ||
                    (flag and EQConstants.FLAG_HUSHEN) == EQConstants.FLAG_SHENSHI) { // 港股通
                if (tags.isNotEmpty()) {
                    tags.append(EQConstants.SIGN_PLUS)
                }
                tags.append(CBAS_OBJ[2])
            } else if ((flag and EQConstants.FLAG_HSGT) == EQConstants.FLAG_HSGT) { // 沪A/深A通(港)
                if (tags.isNotEmpty()) {
                    tags.append(EQConstants.SIGN_PLUS)
                }
                tags.append(CBAS_OBJ[3])
            }

            if (tags.isEmpty()) {
                return CBASConstants.CBAS_IPO_MOREN
            } else {
                return tags.toString()
            }

        }

    }

    private lateinit var mTempTag: TextView
    private lateinit var mRzrqTag: SpecialItemHolder
    private lateinit var mGzyTag: SpecialItemHolder
    private lateinit var mTTag: SpecialItemHolder

    private var mStockInfo: EQBasicStockInfo? = null
    private var mFlag = intArrayOf(-1, -1, -1)
    private var mLastFlag = EQConstants.DEFAULT_STR

    override fun onFinishInflate() {
        super.onFinishInflate()
        mTempTag = findViewById(R.id.stock_temp_tag)
        mRzrqTag = SpecialItemHolder(findViewById(R.id.stock_tag_rzrq))
        mGzyTag = SpecialItemHolder(findViewById(R.id.stock_tag_gzy))
        mTTag = SpecialItemHolder(findViewById(R.id.stock_tag_hsgt))
    }


    fun setStockInfo(stock: EQBasicStockInfo, flagStr: String?) {
        stock.takeIf { it.mStockCode.isNullOrEmpty() ||
                it.mMarket.isNullOrEmpty()}?.let {
            visibility = View.GONE
            return
        }
        if (stock == mStockInfo && TextUtils.equals(flagStr, mLastFlag)) {
            return
        }
        Log.i(LOG_TAG, "flagStr: $flagStr stockCode: ${stock.mStockCode}")
        mStockInfo = stock
        // 处理flag判断有哪些特色标签需要处理
        mLastFlag = flagStr?:EQConstants.DEFAULT_STR
        getShowTagsItem(flagStr?:EQConstants.INVALID_STR_VALUE, mFlag)

        var showShell = false
        // 下面开始初始化处理特色标签，然后再发送http请求特色标签的数据
        if (initSpecialTags()) {
            showShell = true
        }

        val tempStr = getTempTagsTips()
        // 如果有一个标签存在，那么整个外壳试图就要展示
        if (tempStr.isNotEmpty()) {
            mTempTag.text = tempStr
            mTempTag.setTextColor(ThemeManager.getColor(context, R.color.gray_999999))
            mTempTag.visibility = View.VISIBLE
            // 如果有特色标签，这里的间隙要调整一下
            var padBot = 0
            if (showShell) {
                padBot = resources.getDimensionPixelSize(R.dimen.dp_4)
            }
            mTempTag.setPadding(mTempTag.paddingLeft, mTempTag.paddingTop, mTempTag.paddingRight, padBot)
            showShell = true
            Log.i(LOG_TAG, "temp tag need show")
        }

        // 处理外壳视图显示状态
        visibility = if (showShell) {
            View.VISIBLE
        } else {
            View.GONE
        }
    }

    private fun initSpecialTags(): Boolean {
        val reqValue = byteArrayOf(0, 0, 0)
        if (mFlag[0] == EQConstants.FLAG_RZRQ) {
            mRzrqTag.initContext(
                    String.format(RZRQ_STR, EQConstants.DEFAULT_STR, EQConstants.DEFAULT_STR),
                    EQConstants.FLAG_RZRQ)
            reqValue[0] = 1
        } else {
            mRzrqTag.hide()
        }
        if (mFlag[1] == EQConstants.FLAG_GZY) {
            mGzyTag.initContext(
                    String.format(GZY_STR, EQConstants.DEFAULT_STR),
                    EQConstants.FLAG_GZY)
            reqValue[1] = 1
        } else {
            mGzyTag.hide()
        }
        if (mFlag[2] == EQConstants.FLAG_HUSHEN) {
            mTTag.initContext(
                    String.format(T_STR, EQConstants.DEFAULT_STR, EQConstants.DEFAULT_STR),
                    EQConstants.FLAG_HUSHEN)
            reqValue[2] = 1
        } else if (mFlag[2] == EQConstants.FLAG_HSGT && !mStockInfo?.mMarket.isNullOrEmpty()) {
            mTTag.initContext(
                    String.format(T_STR, EQConstants.DEFAULT_STR, EQConstants.DEFAULT_STR),
                    EQConstants.FLAG_HSGT)
            reqValue[2] = 1
        } else {
            mTTag.hide()
        }

        var value: Byte = 0
        val builder = StringBuilder()
        for (v in reqValue) {
            builder.append(v)
            value = value or v
        }

        if (value > 0) {
            request(builder.toString())
        }

        return value > 0
    }

    private fun getTempTagsTips() :String {
        mStockInfo?.mStockName.takeIf {
            !it.isNullOrEmpty() && StockMarketUtils.isHSAG(mStockInfo?.mMarket)
        } ?.let {
            if (it.indexOf(TAG_N) == 0) {
                return resources.getString(R.string.stock_temp_tag_n)
            }
            if (it.indexOf(TAG_XR) == 0) {
                return resources.getString(R.string.stock_temp_tag_xr)
            }
            if (it.indexOf(TAG_XD) == 0) {
                return resources.getString(R.string.stock_temp_tag_xd)
            }
            if (it.indexOf(TAG_DR) == 0) {
                return resources.getString(R.string.stock_temp_tag_dr)
            }
        }
        return EQConstants.INVALID_STR_VALUE
    }

    private fun request(param: String) {
        /*if (mStockInfo?.mStockCode.isNullOrEmpty()) {
            Log.i(LOG_TAG, "invalid stock code to request")
            return
        }
        Observable.create<TagData> {
            val url = String.format(
                    UnifyUrlManager.getInstance().getUrlByResourceId(R.string.stock_tag_url),
                    "${mStockInfo?.mStockCode}_$param")
            Log.i(LOG_TAG, "url: $url")
         *//*   val jsonStr = OkHttpUtils.get().url(url)
                    .build(HxHttpTools.buildOkhttpTcpDnsService())
                    .reqString()*//*
            val jsonStr = ""
            jsonStr?.takeUnless { jsonStr.isNullOrEmpty() }?.let { jstr ->
                try {
                    Log.d(LOG_TAG, "receive-> json: $jstr")
                    val content = JSONObject(jstr)
                    if (content.optInt("status_code", -1) == 0) {
                        val stockCode = content.optString("code")
                        Log.i(LOG_TAG, "receive-> stock:$stockCode")
                        if (stockCode.isNotEmpty() || stockCode == mStockInfo?.mStockCode) {
                            val date = TagData()
                            content.optJSONObject("margin")?.let { margin ->
                                date.rzyeValue = DataUtils.doubleToStrWith2Dec(margin.optDouble("rzye"),
                                        YUAN, EQConstants.DEFAULT_STR)
                                date.rqyeValue = DataUtils.doubleToStrWith2Dec(margin.optDouble("rqye"),
                                        EQConstants.INVALID_STR_VALUE, EQConstants.DEFAULT_STR)
                            }
                            content.optJSONObject("pledge_rights")?.let { pledge ->
                                date.gzyRATIO = pledge.optString("pledge_ratio", EQConstants.DEFAULT_STR)
                            }
                            content.optJSONObject("connect_info")?.let { connect ->
                                date.tValue = DataUtils.doubleToStrWith2Dec(connect.optDouble("inflow"),
                                        YUAN, EQConstants.DEFAULT_STR)
                                date.tRATIO = connect.optString("scale", EQConstants.DEFAULT_STR)
                            }
                            it.onNext(date)
                        }
                    }
                } catch (e: Exception) {
                    Log.i(LOG_TAG, "parse data fail !")
                }
            }
            it.onComplete()
        }.subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).subscribe(object: Observer<TagData>{
            override fun onComplete() {
            }

            override fun onSubscribe(d: Disposable) {
            }

            override fun onNext(t: TagData) {
                Log.i(LOG_TAG, "success: $t")
                if (mRzrqTag.isShow()) {
                    mRzrqTag.updateText(String.format(RZRQ_STR, t.rzyeValue, t.rqyeValue))
                }
                if (mGzyTag.isShow()) {
                    mGzyTag.updateText(String.format(GZY_STR, t.gzyRATIO))
                }
                if (mTTag.isShow()) {
                    mTTag.updateText(String.format(T_STR, t.tRATIO, t.tValue))
                }
            }

            override fun onError(e: Throwable) {
                // 因为一开始就初始化过了，所以这里如果出错了就不用管了
            }
        })*/
    }

    override fun onClick(v: View?) {
       /* mStockInfo?.takeUnless { it.mStockCode.isNullOrEmpty() ||
                it.mMarket.isNullOrEmpty()}?.let {
            when(v?.id) {
                R.id.stock_tag_rzrq -> {
                    val urlPre = UnifyUrlManager.getInstance().getUrlByResourceId(R.string.stock_tag_rzrq)
                    handleJumpEvent(urlPre + it.mStockCode, 0)
                }
                R.id.stock_tag_gzy -> {
                    val url = UnifyUrlManager.getInstance().getUrlByResourceId(R.string.gqzy_url)
                    handleJumpEvent(String.format(url, it.mStockCode, it.mMarket), 1)
                }
                R.id.stock_tag_hsgt -> {
                    val market: String
                    val jumpIndex: Int
                    when {
                        mFlag[2] == EQConstants.FLAG_HUSHEN -> {
                            market = "hk"
                            jumpIndex = 2
                        }
                        mFlag[2] == EQConstants.FLAG_HSGT -> {
                            market = "hs"
                            jumpIndex = 3
                        }
                        else -> return
                    }
                    val url = UnifyUrlManager.getInstance().getUrlByResourceId(R.string.hsgt_fund_stock_url)
                    var code = it.mStockCode
                    if (code?.indexOf(EQConstants.STOCK_STR_HK) == 0) {
                        code = "0" + code.substring(2)
                    }
                    handleJumpEvent(String.format(url, market, code), jumpIndex)
                }
            }
        }*/
    }

    private fun handleJumpEvent(url: String, index: Int) {
        // 发送跳转埋点
       /* //val cbasStr = CBASConstants.CBAS_FENSHI_POP + CBASConstants.CBAS_SPLIT_DIAN + CBAS_OBJ[index]
        var resId: String? = null
        if (CBAS_RES_ID[index] != null) {
            resId = CBAS_RES_ID[index] + mStockInfo?.mStockCode
        }
        val gotoModel = CbasJumpPageModel(ProtocalDef.FRAMEID_COMMON_BROWSER.toString(),
                null, resId)
        HexinCBASUtil.sendStandardJumpPageCbas(UserBehaviorAnalysis.ACTION_OPERATE_TYPE_CLICK,
                cbasStr, false, null, mStockInfo, gotoModel)
        // 跳转活动的落地页
        ClientHtmlJump.jumpToUrl(url, null, ProtocalDef.FRAMEID_COMMON_BROWSER)*/
    }

    private fun getShowTagsItem(flagStr: String, flagValue: IntArray) {
        val flag: Int
        try {
            flag = Integer.valueOf(flagStr.trim())
        } catch (e: Exception) {
            Log.printStackTrace(e)
            return
        }

        if ((flag and EQConstants.FLAG_RZRQ) == EQConstants.FLAG_RZRQ) { //融资融券标志位
            flagValue[0] = EQConstants.FLAG_RZRQ
        }
        if ((flag and EQConstants.FLAG_GZY) == EQConstants.FLAG_GZY) { // 高质押
            flagValue[1] = EQConstants.FLAG_GZY
        }
        if ((flag and EQConstants.FLAG_HUSHEN) == EQConstants.FLAG_HUSHEN ||
                (flag and EQConstants.FLAG_HUSHEN) == EQConstants.FLAG_HUSHI ||
                (flag and EQConstants.FLAG_HUSHEN) == EQConstants.FLAG_SHENSHI) { // 港股通
            flagValue[2] = EQConstants.FLAG_HUSHEN
        } else if ((flag and EQConstants.FLAG_HSGT) == EQConstants.FLAG_HSGT) { // 沪A/深A通(港)
            flagValue[2] = EQConstants.FLAG_HSGT
        }
    }

    inner class SpecialItemHolder(shellView: RelativeLayout) {

        var icon: TextView
        var value: TextView
        var arrow: ImageView
        var container: RelativeLayout = shellView

        init {
            container.setOnClickListener(this@StockTagView)
            icon = container.findViewById(R.id.stock_tag_icon)
            value = container.findViewById(R.id.stock_tag_value)
            value.setTextColor(ThemeManager.getColor(context, R.color.gray_999999))
            arrow = container.findViewById(R.id.stock_tag_arrow)
            arrow.setBackgroundResource(ThemeManager.getDrawableRes(context, R.drawable.right_forward))
        }

        fun initContext(text: CharSequence, flagValue: Int) {
            TagTextViewHelper.updateTextViewByType(context, icon, mStockInfo?.mMarket, flagValue)
            value.text = text
            container.visibility = View.VISIBLE
        }

        fun updateText(text: CharSequence) {
            value.text = text
        }

        fun hide() {
            container.visibility = View.GONE
        }

        fun isShow(): Boolean {
            return container.visibility == View.VISIBLE
        }

    }

    inner class TagData {
        var rzyeValue: String = EQConstants.DEFAULT_STR
        var rqyeValue: String = EQConstants.DEFAULT_STR
        var gzyRATIO: String = EQConstants.DEFAULT_STR
        var tValue: String = EQConstants.DEFAULT_STR
        var tRATIO: String = EQConstants.DEFAULT_STR

        override fun toString(): String {
            return "rz:$rzyeValue rq:$rqyeValue gzy:$gzyRATIO tv:$tValue tr:$tRATIO"
        }

    }

}
