package com.hexin.android.component.hangqing

import android.content.Context
import android.graphics.Rect
import android.text.TextUtils
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.hexin.android.component.common.BaseLinearComponent
import com.hexin.android.component.hangqing.hushen.ZTAnalysisModel
import com.hexin.android.component.model.AndroidColumnDragableTableModel
import com.hexin.android.service.UnifyUrlManager
import com.hexin.android.theme.ThemeManager
import com.hexin.android.view.ShadowDrawable
import com.hexin.android.view.TagTextViewHelper
import com.hexin.app.event.EQEvent
import com.hexin.app.event.action.EQGotoUnknownFrameAction
import com.hexin.app.event.param.EQGotoParam
import com.hexin.app.event.param.EQParam
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.middleware.GlobalHandler
import com.hexin.middleware.MiddlewareProxy
import com.hexin.middleware.ProtocalDef
import com.hexin.middleware.http.IRequest
import com.hexin.plat.android.R
import com.hexin.util.HexinThreadPool
import com.hexin.util.HexinUtils
import com.hexin.util.Log
import com.hexin.util.ViewUtils
import com.hexin.util.config.EQConstants
import org.json.JSONException
import org.json.JSONObject

class HangqingZTAnalysis(context: Context?, attrs: AttributeSet?) : BaseLinearComponent(context, attrs), View.OnClickListener {
    companion object {
        private const val TAG = "HangqingZTAnalysis"
        private const val HANGQING_ZHANGDIE = 34387
        fun isShowZTAnalysis(): Boolean {
            return MiddlewareProxy.getmRuntimeDataManager()?.isShowZTAnalysis ?: false
        }
        fun setIsShowZTAnalysis(isShow: Boolean) {
            MiddlewareProxy.getmRuntimeDataManager()?.setIsShowZTAnalysis(isShow)
        }
    }

    private val mCollapseTv by lazy { findViewById<TextView>(R.id.analysis_collapse_text) }
    private val mCollapseIcon by lazy { findViewById<ImageView>(R.id.analysis_collapse_icon) }
    private val mHeaderCurrentPrice by lazy { findViewById<TextView>(R.id.analysis_header_current_price) }
    private val mHeaderIncrease by lazy { findViewById<TextView>(R.id.analysis_header_increase) }
    private val mHeaderNewZhangDie by lazy { findViewById<TextView>(R.id.analysis_header_zhangdie) }
    private val mRecyclerView by lazy { findViewById<RecyclerView>(R.id.analysis_stock_list) }
    private val mAnalysisMap = HashMap<String?, ZTAnalysisModel>()
    private val mItemSpace = resources.getDimensionPixelOffset(R.dimen.dp_4)
    var stockTable: HangqingMarketRobotTable? = null


    override fun onFinishInflate() {
        super.onFinishInflate()
        findViewById<View>(R.id.analysis_collapse_container).setOnClickListener(this)
        mRecyclerView.adapter = AnalysisAdapter()
        mRecyclerView.layoutManager = LinearLayoutManager(context)
        mRecyclerView.addItemDecoration(SpaceDecoration())
        mRecyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
                if (newState == RecyclerView.SCROLL_STATE_IDLE) {
                    val layoutManager = recyclerView.layoutManager as LinearLayoutManager
                    val firstPosition = layoutManager.findFirstVisibleItemPosition()
                    val lastPosition = layoutManager.findLastVisibleItemPosition()
                    stockTable?.requestStockData(firstPosition, lastPosition)
                }
            }
        })
    }

    override fun initTheme() {
        this.setBackgroundColor(ThemeManager.getColor(context, R.color.white_FFFFFF))
        mCollapseTv.setTextColor(ThemeManager.getColor(context, R.color.gray_666666))
        mCollapseIcon.setImageResource(R.drawable.analysis_expand)
        val textColor = ThemeManager.getColor(context, R.color.gray_999999)
        mHeaderCurrentPrice.setTextColor(textColor)
        mHeaderIncrease.setTextColor(textColor)
        mHeaderNewZhangDie.setTextColor(textColor)
        mRecyclerView.adapter?.notifyDataSetChanged()
    }

    override fun onClick(v: View?) {
        if (v == null) {
            return
        }
        when(v.id) {
            R.id.analysis_collapse_container -> {
                stockTable?.showTable((mRecyclerView.layoutManager as LinearLayoutManager).findFirstVisibleItemPosition())
            }
            R.id.stock_list_item_container, R.id.stock_list_zt_reason -> {
                val viewHolder = v.tag as AnalysisViewHolder
                val stockInfo = viewHolder.stockInfo
                if (stockInfo.isStockCodeValiable && stockInfo.isStockNameValiable && stockInfo.isMarketIdValiable ) {
                    stockTable?.model?.saveTitleStock(viewHolder.adapterPosition, true)
                    // 发送查看股票信息请求
                    val gotoFrameAction = EQGotoUnknownFrameAction(EQEvent.EVENT_TYPE_CLICK, ProtocalDef.FRAMEID_STOCK_GG_PRICE, EQGotoUnknownFrameAction.SWITCH_STOCK_GROUP, stockInfo.mMarket)
                    val gotoParam = EQGotoParam(EQParam.PARAM_TYPE_STOCKINFO, stockInfo)
                    gotoParam.setUsedForAll()
                    gotoFrameAction.param = gotoParam
                    MiddlewareProxy.executorAction(gotoFrameAction)
                }
            }
        }
    }

    fun scrollTo(position: Int) {
        //直接用recyclerView.scrollToPosition时，若对应的view可见，则该view会显示在底部
        (mRecyclerView.layoutManager as LinearLayoutManager).scrollToPositionWithOffset(position, 0)
    }

    fun requestData() {
        HexinThreadPool.getThreadPool().execute {
            val url = UnifyUrlManager.getInstance().getUrlByResourceId(R.string.zhangting_url)
            val response = IRequest.post(url).execute()
            val result = parseAnalysisData(response)
            GlobalHandler.post {
                if (result != null && !result.isEmpty()) {
                    mAnalysisMap.clear()
                    // 这里已经保证了result中的每一个数据都是有效的,即stockCode不为空
                    for (model in result) {
                        mAnalysisMap[(model.stockCode)] = model
                    }
                    mRecyclerView.adapter?.notifyDataSetChanged()
                }
            }
        }
    }

    private fun parseAnalysisData(resultStr: String?): List<ZTAnalysisModel>? {
        if (TextUtils.isEmpty(resultStr)) {
            return null
        }
        val result = ArrayList<ZTAnalysisModel>()
        return try {
            val jsonObject = JSONObject(resultStr)
            val errorNo = jsonObject.optString(ZTAnalysisModel.JSON_KEY_ERROR_NO)
            if (ZTAnalysisModel.JSON_DATA_RIGHT_FLAG != errorNo) {
                // 数据无效
                val errorInfo = jsonObject.optString(ZTAnalysisModel.JSON_KEY_ERROR_INFO)
                GlobalHandler.post {
//                    view.showToast(errorInfo)
                }
                return null
            }
            val jsonArray = jsonObject.optJSONArray(ZTAnalysisModel.JSON_KEY_STOCKS)
            var analysisModel: ZTAnalysisModel?
            for (i in 0 until jsonArray.length()) {
                analysisModel = parseAnalysisModel(jsonArray.optJSONObject(i))
                if (analysisModel?.isValid() == true) {
                    result.add(analysisModel)
                }
            }
            result
        } catch (jsonException: JSONException) {
            Log.printStackTrace(jsonException)
            Log.e(TAG, "parse error")
            null
        }
    }

    private fun parseAnalysisModel(jsonObject: JSONObject): ZTAnalysisModel? {
        try {
            val tagArray = jsonObject.optJSONArray(ZTAnalysisModel.JSON_KEY_STOCK_LABELS)
            val tagList: List<String>?
            if (tagArray != null) {
                tagList = ArrayList()
                for (i in 0 until Math.min(ZTAnalysisModel.MAX_SHOW_TAGS, tagArray.length())) {
                    tagList.add(tagArray.getJSONObject(i).getString(ZTAnalysisModel.JSON_KEY_LABEL_NAME))
                }
            } else {
                tagList = null
            }
            val description = jsonObject.optString(ZTAnalysisModel.JSON_KEY_DESCRIPTION)
            return ZTAnalysisModel(
                    jsonObject.optString(ZTAnalysisModel.JSON_KEY_STOCK_CODE),
                    jsonObject.optString(ZTAnalysisModel.JSON_KEY_STOCK_NAME),
                    description?.replace("\n", EQConstants.DESCRIPTION_EMPTY_STRING), tagList
            )
        } catch (jsonException: JSONException) {
            Log.printStackTrace(jsonException)
            Log.e(TAG, "parse error")
            return null
        }
    }

    fun forceStopScroll() {
        ViewUtils.forceStopRecyclerViewScroll(mRecyclerView)
    }

    fun refreshList() {
        mRecyclerView.adapter?.notifyDataSetChanged()
    }

    inner class AnalysisViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        private val nameView by lazy { itemView.findViewById<TextView>(R.id.stock_list_name) }
        private val codeView by lazy { itemView.findViewById<TextView>(R.id.stock_list_code) }
        private val logoView by lazy { itemView.findViewById<TextView>(R.id.stock_list_market_logo) }
        private val currentPriceView by lazy { itemView.findViewById<TextView>(R.id.stock_list_current_price) }
        private val increaseView by lazy { itemView.findViewById<TextView>(R.id.stock_list_increase) }
        private val zhangdieView by lazy { itemView.findViewById<TextView>(R.id.stock_list_zhangdie) }
        private val ztReasonTv by lazy {itemView.findViewById<TextView>(R.id.stock_list_zt_reason)}
        private val stockTags by lazy {arrayOf<TextView>(itemView.findViewById(R.id.stock_list_tag1),
                itemView.findViewById(R.id.stock_list_tag2),
                itemView.findViewById(R.id.stock_list_tag3))}
        val stockInfo by lazy { EQBasicStockInfo() }

        private val shadowDrawable by lazy { ShadowDrawable(resources.getDimensionPixelOffset(R.dimen.dp_4), resources.getDimensionPixelOffset(R.dimen.dp_4)) }
        init {
            itemView.setBackgroundDrawable(shadowDrawable)
            itemView.setOnClickListener(this@HangqingZTAnalysis)
            itemView.tag = this
            ztReasonTv.setOnClickListener(this@HangqingZTAnalysis)
            ztReasonTv.tag = this
        }

        private fun hideAllTag() {
            for (tag in stockTags) {
                tag.visibility = View.GONE
            }
        }

        private fun getValue(model: AndroidColumnDragableTableModel, position: Int, id: Int): String {
            val value = model.getValueById(position, id)
            return if (TextUtils.isEmpty(value)) {
                EQConstants.INVALID_STR_VALUE
            } else {
                value
            }
        }

        private fun getColor(model: AndroidColumnDragableTableModel, position: Int, id: Int): Int {
            val color = model.getColorById(position, id)
            return HexinUtils.getTransformedColor(color, context)
        }

        private fun getStockTag(model: AndroidColumnDragableTableModel, position: Int): Int {
            val tagValue = getValue(model, position, ProtocalDef.HANGQING_STOCK_RZRQ)
            return try {
                tagValue.toInt()
            } catch (exception: Exception) {
                Log.printStackTrace(exception)
                Log.e(TAG, "translate stock tag error")
                EQConstants.INVALID
            }
        }

        fun setData(position: Int) {
            val model = stockTable?.model
            if (model == null) {
                showDefaultView()
            } else {
                val pos = model.getScrollPos()
                val index = if (pos <= 0) position else position - pos
                if (index >= 0 && index < model.rows) {
                    stockInfo.mStockCode = getValue(model, index, ProtocalDef.HANGQING_STOCK_CODE_4)
                    stockInfo.mStockName = getValue(model, index, ProtocalDef.HANGQING_STOCK_NAME)
                    stockInfo.mMarket = getValue(model, index, ProtocalDef.HANGQING_STOCK_MARKETID)

                    nameView.text = stockInfo.mStockName
                    nameView.setTextColor(ThemeManager.getColor(context, R.color.gray_323232))

                    codeView.text = stockInfo.mStockCode
                    codeView.setTextColor(ThemeManager.getColor(context, R.color.gray_999999))

                    val stockType = TagTextViewHelper.getStockTypeLogo(getValue(model, index, ProtocalDef.HANGQING_STOCK_RZRQ), "")
                    TagTextViewHelper.buildTagTextView(context, logoView, stockType)


                    currentPriceView.text = getValue(model, index, ProtocalDef.HANGQING_STOCK_PRICE)
                    currentPriceView.setTextColor(getColor(model, index, ProtocalDef.HANGQING_STOCK_PRICE))

                    zhangdieView.text = getValue(model, index, HANGQING_ZHANGDIE)
                    zhangdieView.setTextColor(getColor(model, index, HANGQING_ZHANGDIE))

                    increaseView.text = getValue(model, index, ProtocalDef.HANGQING_AG_ZHANGFU)
                    increaseView.setTextColor(getColor(model, index, ProtocalDef.HANGQING_AG_ZHANGFU))
                    setAnalysisData(stockInfo.mStockCode)
                } else {
                    showDefaultView()
                }
            }
        }

        private fun showDefaultView() {
            val color323232 = ThemeManager.getColor(context, R.color.gray_323232)
            logoView.visibility = View.GONE
            nameView.text = EQConstants.DEFAULT_STR
            nameView.setTextColor(color323232)
            codeView.text = EQConstants.DEFAULT_STR
            codeView.setTextColor(ThemeManager.getColor(context, R.color.gray_999999))
            currentPriceView.text = EQConstants.DEFAULT_STR
            currentPriceView.setTextColor(color323232)
            increaseView.text = EQConstants.DEFAULT_STR
            increaseView.setTextColor(color323232)
            zhangdieView.text = EQConstants.DEFAULT_STR
            zhangdieView.setTextColor(color323232)
            ztReasonTv.visibility = View.GONE
            hideAllTag()
        }

        private fun setAnalysisData(stockCode: String) {
            shadowDrawable.setColor(ThemeManager.getColor(context, R.color.shadow))
            val reasonModel = mAnalysisMap[stockCode]
            if (reasonModel == null) {
                ztReasonTv.visibility = View.GONE
                hideAllTag()
            } else {
                ztReasonTv.visibility = View.VISIBLE
                ztReasonTv.text = reasonModel.ztReason
                ztReasonTv.setTextColor(ThemeManager.getColor(context, R.color.gray_666666))
                if (reasonModel.tags != null && !reasonModel.tags.isEmpty()) {
                    for ((index, tag) in reasonModel.tags.withIndex()) {
                        stockTags[index].visibility = View.VISIBLE
                        stockTags[index].text = tag
                        stockTags[index].setTextColor(ThemeManager.getColor(context, R.color.red_E93030))
                        stockTags[index].setBackgroundResource(ThemeManager.getDrawableRes(context, R.drawable.stock_tag_bg))
                    }
                    for (i in reasonModel.tags.size until stockTags.size) {
                        stockTags[i].visibility = View.INVISIBLE
                    }
                } else {
                    hideAllTag()
                }
            }
        }
    }

    inner class AnalysisAdapter: RecyclerView.Adapter<AnalysisViewHolder>() {
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): AnalysisViewHolder {
            return AnalysisViewHolder(LayoutInflater.from(context).
                    inflate(R.layout.item_zt_analysis, parent, false))
        }

        override fun getItemCount(): Int {
            val model = stockTable?.model
            return when {
                model == null -> 0
                model.totalSize <= 0 -> model.rows
                else -> model.totalSize
            }
        }

        override fun onBindViewHolder(holder: AnalysisViewHolder, position: Int) {
            holder.setData(position)
        }
    }

    private inner class SpaceDecoration : RecyclerView.ItemDecoration() {
        override fun getItemOffsets(outRect: Rect, view: View, parent: RecyclerView, state: RecyclerView.State) {
            super.getItemOffsets(outRect, view, parent, state)
            val position = parent.getChildLayoutPosition(view)
            if (position > 0) {
                outRect.top = mItemSpace
            }
        }
    }
}