package com.hexin.android.component.hangqing.hangqingfunds

import com.hexin.android.component.curve.data.CurveObj
import com.hexin.android.component.curve.data.CurveUtilTool
import com.hexin.android.ui.NetWorkClinet
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.middleware.MiddlewareProxy
import com.hexin.middleware.ProtocalDef
import com.hexin.middleware.ProtocalDef.SM_HQ_PREVCLOSE
import com.hexin.middleware.QueueManagement
import com.hexin.middleware.data.StuffBaseStruct
import com.hexin.middleware.data.mobile.StuffCurveStruct
import com.hexin.util.Log

class FenshiClient(val listener: OnFenshiDataUpdateListener) : NetWorkClinet {
    companion object {
        private const val FENSHI_REQUEST_TEXT = "\r\nstockcode=%s\r\nmarketcode=%s"
        private const val FENSHI_VALUE_DATA_ID = 10
        private const val FENSHI_REQUEST_PAGE_ID = 1229
        private const val TAG = "FenshiClient"
    }
    private var mRequestText: String? = null
    private var mLastData: StuffCurveStruct? = null
    var stockInfo: EQBasicStockInfo? = null
    set(value) {
        mLastData = null
        mRequestText = if (value == null || !value.isStockCodeValiable || !value.isMarketIdValiable) null else
            String.format(FENSHI_REQUEST_TEXT, value.mStockCode, value.mMarket)
        removeRequest()
        field = value
    }

    override fun receive(struct: StuffBaseStruct) {
        if (struct !is StuffCurveStruct) {
            Log.i(TAG, "fenshi receive struct = $struct, request = $mRequestText")
            return
        }
        mLastData = CurveUtilTool.mergeFile(struct, mLastData)
        if (mLastData == null) {
            Log.i(TAG, "fenshi receive merge data = null")
            return
        }
        val prePrices = mLastData!!.getExtData(SM_HQ_PREVCLOSE)
        val currentPrices = mLastData!!.getData(FENSHI_VALUE_DATA_ID)
        val result = if (prePrices !is Double) {
            Log.i(TAG, "fenshi receive prePrices is not double")
            null
        } else {
            val zdValues = if (currentPrices == null) null else {
                val values = ArrayList<Float>(currentPrices.size)
                for (currentPrice in currentPrices) {
                    if (!CurveObj.isInvalidData(currentPrice)) {
                        values.add((currentPrice - prePrices).toFloat())
                    }
                }
                values
            }
            FenshiModule(zdValues, prePrices.toFloat())
        }
        listener.onFenshiDataUpdate(result)
    }

    override fun request() {
        requestData(false)
    }

    fun requestData(isFlush: Boolean) {
        mRequestText?.let {
            MiddlewareProxy.request(ProtocalDef.INDEX_BAR_FRAME_ID, FENSHI_REQUEST_PAGE_ID, QueueManagement.getInstanceId(this), it,
                    true, true)
        }
    }

    fun removeRequest() {
        QueueManagement.remove(this)
    }

    data class FenshiModule(val zdValues: List<Float>?, val prePrice: Float)
    interface OnFenshiDataUpdateListener {
        fun onFenshiDataUpdate(data: FenshiModule?)
    }
}
