package com.hexin.android.component.hangqing.hangqingfunds

import android.content.Context
import android.graphics.*
import android.os.Build
import android.util.AttributeSet
import android.view.View
import com.hexin.android.component.yidong.dpbidyd.LineChartAxisConfig
import com.hexin.android.component.yidong.dpbidyd.LineModel
import com.hexin.plat.android.R
import com.hexin.android.theme.ThemeManager
import com.hexin.util.config.EQConstants

/**
 * 折线图支持：
 *
 * 1.全部值必须为正值
 * 2.左右两侧双Y轴
 * 3.折线阴影
 *
 * 目前支持的线条类型比较少,所以draw的具体实现在该类中,后续如果有拓展,请把draw的工作通过
 * 传Canvas、绘制区域、转换系数 交个不同的LineModel去做
 *
 * Create by likai@myhexin.com on 2018-11-28
 */

class FundsLineChartView(context: Context?, attrs: AttributeSet?) : View(context, attrs) {

    companion object {
        const val TAG = "LineChartView"
        const val Y_TEXT_SPACE = 8 // Y坐标文字与线的距离
        const val MIN_LIMIT_Y = 0.5f // 数据里的最小值小于这个数时,Y坐标的最小值将为0
        const val TEXT_RIGHT_FORMAT = "%.2f亿"
        const val TEXT_LEFT_FORMAT = "%.2f%%"
    }

    private var mConfig = LineChartAxisConfig(
        LineChartAxisConfig.MIN_VALUE_COUNT,
            LineChartAxisConfig.MIN_HOR_LINE_COUNT, LineChartAxisConfig.MIN_VER_LINE_COUNT
    ) // 画坐标轴用到的配置参数
    // 下面的参数都是存的宽高位置值
    private val mDrawArea = RectF(0f, 0f, 0f, 0f) // 能用来绘制的区域
    private lateinit var mXTextPoint: Array<Float> // 所有X轴下方文本绘制的锚点位置
    private lateinit var mXPoint: Array<Float> // 画竖直线的X坐标
    private lateinit var mYPoint: Array<Float> // 画水平横线的Y坐标,同时也作为画Y方向坐标值的锚点
    // 绘制用的画笔
    private lateinit var mPaint: Paint
    private lateinit var mLinePaint: Paint // 画线专用的画笔

    private val mLinesOnLeftAxis = arrayListOf<LineModel>() // 以左侧Y轴为基准画的折线
    private val mLinesOnRightAxis = arrayListOf<LineModel>() // 以右侧Y轴为基准画的折线
    // 下面参数都是存放的数值,表示实际的值
    private var mMinLeftY = 0f // 左侧Y坐标与X轴相交原点(在View的左下方)的数值
    private var mGapLeftY = 0f // 左侧每一根线之间的Y坐标差值,大于0时才能画
    private var mMinRightY = 0f // 右侧Y坐标与X轴相交原点(在View的右下方)的数值
    private var mGapRightY = 0f // 右侧每一根线之间的Y坐标差值,大于0时才能画
    // 下面的系数用来把一个实际数值转换成在View的Canvas中对应的位置
    private var mLeftYCoeVToH = 0f // 数值在左侧Y坐标轴里表示的高度系数 数值 * mLeftYCoeVToH = 左侧Y轴坐标
    private var mRightYCoeVToH = 0f // 数值在右侧Y坐标轴里表示的高度系数 数值 * mRightYCoeVToH = 右侧Y轴坐标
    private var mXStep = 0f // 折线的点与点之间的X方向间距
    private val mArrowSize = getContext().resources.getDimensionPixelSize(R.dimen.dp_6)
    private val pathEffect = DashPathEffect(floatArrayOf(8f, 8f), 2f)
    var fundsType: HangQingFundsView.FundsType = HangQingFundsView.FundsType.HS
    var fundsTips: String? = null
    var leftBaseValue: Float = 1f

    override fun onFinishInflate() {
        super.onFinishInflate()
        // android4.3改为软件加速,防止出现绘制问题
        if (Build.VERSION_CODES.JELLY_BEAN_MR2 == Build.VERSION.SDK_INT) {
            setLayerType(LAYER_TYPE_SOFTWARE, null)
        }
        mPaint = Paint()
        mPaint.isAntiAlias = true
        mPaint.style = Paint.Style.FILL

        mLinePaint = Paint()
        mLinePaint.isAntiAlias = true
    }

    fun configAxis(config: LineChartAxisConfig) {
        mConfig = config
    }

    fun updateView() {
        postInvalidate()
    }

    /**
     * 添加以左侧Y轴为参考的曲线,如果给null不生效,想清空折线可传
     * new ArrayList<LineModel>()
     *
     * 添加完成之后需要自行调用 updateView() 更新取消图
     */
    fun addLeftLines(models: List<LineModel>?) {
        models?.let {
            addLines(true, it)
        }
    }

    /**
     * 添加以右侧Y轴为参考的曲线,如果给null不生效,想清空折线可传
     * new ArrayList<LineModel>()
     *
     * 添加完成之后需要自行调用 updateView() 更新取消图
     */
    fun addRightLines(models: List<LineModel>?) {
        models?.let {
            addLines(false, it)
        }
    }

    private fun addLines(isLeft: Boolean, models: List<LineModel>) {
        if (models.isEmpty()) {
            if (isLeft) {
                mMinLeftY = 0f
                mGapLeftY = 0f
                mLinesOnLeftAxis.clear()
            } else {
                mMinRightY = 0f
                mGapRightY = 0f
                mLinesOnRightAxis.clear()
            }
        } else {
            val limitArray = arrayListOf<Float>()
            // 这一个循环找到所有曲线里的最大和最小值
            var maxInModel:Float?
            var minInModel:Float?
            for (model in models) {
                maxInModel = model.values.max()
                minInModel = model.values.min()
                if (maxInModel == null || minInModel == null) {
                    return
                }
                model.max = maxInModel
                model.min = minInModel
                limitArray.add(model.max)
                limitArray.add(model.min)
            }
            val max = limitArray.max()
            val min = limitArray.min()
            if (max == null || min == null || max < min) {
                return
            }
            val maxY = Math.max(Math.abs(max), Math.abs(min))
            val minY = (-maxY / MIN_LIMIT_Y).toInt() * MIN_LIMIT_Y - MIN_LIMIT_Y
            val gapY = -2 * minY / mConfig.horLineCount
            if (isLeft) {
                mMinLeftY = minY
                mGapLeftY = gapY
                mLinesOnLeftAxis.clear()
                mLinesOnLeftAxis.addAll(models)
            } else {
                mMinRightY = minY
                mGapRightY = gapY
                mLinesOnRightAxis.clear()
                mLinesOnRightAxis.addAll(models)
            }
        }

        calculateXYPoint()
        calculateCoefficient()
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        mDrawArea.set(paddingLeft.toFloat(), paddingTop.toFloat() + mArrowSize,
                (measuredWidth - paddingRight).toFloat(), (measuredHeight - paddingBottom).toFloat())
        calculateXYPoint()
        calculateCoefficient()
    }

    /**
     * 计算所有X,Y基本点的位置
     */
    private fun calculateXYPoint() {
        // 先把X方向的文本位置算完
        var size = mConfig.xText?.size?:0
        if (size > 0) {
            mXTextPoint = Array(size) {0f}
            val gapX = (mDrawArea.right - mDrawArea.left - mConfig.xTextHPadding * 2) / (size - 1)
            for (i in 0 until size) {
                mXTextPoint[i] = mDrawArea.left + gapX * i + mConfig.xTextHPadding
            }
        }
        size = mConfig.verLineCount
        if (size > 0) {
            mXPoint = Array(size) {0f}
            val gapX = (mDrawArea.right - mDrawArea.left) / (size - 1)
            for (i in 0 until size) {
                mXPoint[i] = mDrawArea.left + gapX * i
            }
        }

        // 再来计算画那几根横线的Y坐标
        mYPoint = Array(mConfig.horLineCount + 1) {0f} // 这里+1是加了一根X轴
        // 最后一条线的位置
        val bottomLineY = mDrawArea.bottom - mConfig.xTextSize - mConfig.xTextVPadding
        // 每道线之间的距离
        val gapY = (bottomLineY - mDrawArea.top) / mConfig.horLineCount
        // 画线是从下往上画
        mYPoint[0] = bottomLineY
        for (i in 1..mConfig.horLineCount) {
            mYPoint[i] = bottomLineY - gapY * i
        }
    }

    /**
     * 计算转换系数, 每单位的数值能换算为多少高度
     */
    private fun calculateCoefficient() {
        mLeftYCoeVToH = if (mGapLeftY == 0f) 0f
        else {
            (mYPoint[0] - mYPoint[mConfig.horLineCount] - mConfig.horLineWidth * 2) / (mGapLeftY * mConfig.horLineCount)
        }

        mRightYCoeVToH = if (mGapRightY == 0f) 0f
        else {
            (mYPoint[0] - mYPoint[mConfig.horLineCount] - mConfig.horLineWidth * 2) / (mGapRightY * mConfig.horLineCount)
        }

        mXStep = if (mConfig.valueCount - 1 == 0) 0f
        else {
            (mDrawArea.right - mDrawArea.left - mConfig.xPadding * 2) / (mConfig.valueCount - 1)
        }
    }


    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        if (mDrawArea.right - mDrawArea.left <= 0 || mDrawArea.bottom - mDrawArea.top <= 0) {
            return // 没有绘制空间了
        }
        drawXText(canvas)
        drawAxisLine(canvas)
        drawLine(canvas, mLeftYCoeVToH, mMinLeftY, mLinesOnLeftAxis)
        drawLine(canvas, mRightYCoeVToH, mMinRightY, mLinesOnRightAxis)
        drawYText(canvas)
    }

    /**
     * 画X轴下面的文本
     */
    private fun drawXText(canvas: Canvas?) = mConfig.xText?.let{
        mPaint.color = mConfig.xTextColor
        mPaint.textSize = mConfig.xTextSize
        val metrics = mPaint.fontMetrics
        for (i in 0 until mXTextPoint.size) {
            mPaint.textAlign = when (i) {
                0 -> Paint.Align.LEFT
                mXTextPoint.size - 1 -> Paint.Align.RIGHT
                else -> Paint.Align.CENTER
            }
            canvas?.drawText(it[i], mXTextPoint[i], mYPoint[0] - metrics.ascent + mConfig.xTextVPadding, mPaint)
        }
    }

    /**
     * 画坐标轴横线
     */
    private fun drawAxisLine(canvas: Canvas?) {
        mPaint.color = mConfig.horLineColor
        mPaint.strokeWidth = mConfig.horLineWidth
        mPaint.style = Paint.Style.STROKE
        val path = Path()
        var offset:Int
        val arrowX = (mDrawArea.right - mDrawArea.left) / 4 *
                if (fundsType == HangQingFundsView.FundsType.HS) 1 else 3
        for (i in 0 until mYPoint.size) {
            path.reset()
            offset = 0
            if (i == 0 || i == mYPoint.size - 1) {
                mPaint.pathEffect = null
                if (i == mYPoint.size - 1) {
                    offset = 1
                }
            } else {
                mPaint.pathEffect = pathEffect
            }
            path.moveTo(mDrawArea.left, mYPoint[i] + offset)
            if (i == mYPoint.size - 1) {
                path.lineTo(arrowX - mArrowSize, mYPoint[i])
                path.lineTo(arrowX, mYPoint[i] - mArrowSize)
                path.lineTo(arrowX + mArrowSize, mYPoint[i])
            }
            path.lineTo(mDrawArea.right, mYPoint[i] + offset)
            canvas?.drawPath(path, mPaint)
        }
        for (i in 0 until mXPoint.size) {
            path.reset()
            offset = 0
            if (i == 0 || i == mConfig.verLineCount - 1) {
                if (i == 0) {
                    offset = 1
                } else {
                    offset = -1
                }
                mPaint.pathEffect = null
            } else {
                mPaint.pathEffect = pathEffect
            }
            path.moveTo(mXPoint[i] + offset, mDrawArea.top)
            path.lineTo(mXPoint[i] + offset, mYPoint[0])
            canvas?.drawPath(path, mPaint)
        }
        mPaint.pathEffect = null
        mPaint.style = Paint.Style.FILL
    }

    /**
     * 画Y轴的文本
     */
    private fun drawYText(canvas: Canvas?) {
        mPaint.textSize = mConfig.yTextSize
        // 画左边的Y坐标
        if (mGapLeftY > 0) {
            mPaint.textAlign = Paint.Align.LEFT
            drawYText(canvas, true)
        }

        // 画右边的Y坐标
        if (mGapRightY > 0) {
            mPaint.textAlign = Paint.Align.RIGHT
            drawYText(canvas, false)
        }
        fundsTips?.let {
            mPaint.color = ThemeManager.getColor(context, R.color.gray_999999)
            mPaint.textAlign = Paint.Align.CENTER
            canvas?.drawText(it, (mDrawArea.right - mDrawArea.left) / 2,
                mYPoint[0] - mPaint.fontMetrics.descent - Y_TEXT_SPACE, mPaint)
        }
    }

    private fun drawYText(canvas: Canvas?, isLeft: Boolean) {
        var yPoint: Float
        var yValue: Float
        var yText: String
        val x = if (isLeft) mDrawArea.left + Y_TEXT_SPACE else mDrawArea.right - Y_TEXT_SPACE
        val formatStr = if (isLeft) TEXT_LEFT_FORMAT else TEXT_RIGHT_FORMAT
        for (i in 0..mConfig.horLineCount) {
            yPoint = if (i == 0) {
                mYPoint[i] - mPaint.fontMetrics.descent - Y_TEXT_SPACE
            } else {
                mYPoint[i] - mPaint.fontMetrics.ascent + Y_TEXT_SPACE
            }
            yValue = if (isLeft) (mMinLeftY + mGapLeftY * i) / leftBaseValue * 100
            else mMinRightY + mGapRightY * i
            yText = String.format(formatStr, yValue)
            mPaint.color =
            when {
                yValue > 0.01f -> {
                    yText = EQConstants.SIGN_PLUS.plus(yText)
                    ThemeManager.getColor(context, R.color.red_E93030)
                }
                yValue < -0.01f -> {
                    ThemeManager.getColor(context, R.color.green_009900)
                }
                else -> {
                    ThemeManager.getColor(context, R.color.gray_323232)
                }
            }
            canvas?.drawText(yText, x, yPoint, mPaint)
        }
    }

    private fun drawLine(canvas: Canvas?, coe: Float, minY: Float, list: List<LineModel>?) {
        list?.let {
            val path = Path()
            val startX = mDrawArea.left + mConfig.horLineWidth
            var pointCount = mConfig.valueCount

            for (model in it) {
                if (model.values.isNotEmpty()) {
                    // 1.准备画笔
                    mLinePaint.strokeWidth = model.lineWidth.toFloat()
                    mLinePaint.style = Paint.Style.STROKE
                    mLinePaint.color = model.lineColor
                    // 2.开始组建折线的Path
                    path.reset()
                    path.moveTo(startX, mYPoint[0] - (model.values[0] - minY) * coe - mConfig.horLineWidth)
                    pointCount = Math.min(model.values.size, pointCount)
                    for (i in 1 until pointCount) {
                        path.lineTo(startX + i * mXStep,
                                mYPoint[0] - (model.values[i] - minY) * coe - mConfig.horLineWidth)
                    }
                    // 3.绘制折线
                    if (!model.isSolid) {
                        mLinePaint.pathEffect = pathEffect
                    }
                    canvas?.drawPath(path, mLinePaint)
                    mLinePaint.pathEffect = null
                    if (model.hasShadow) {
                        val shadow = Path()
                        // 4.把折线加进来
                        shadow.addPath(path)
                        // 最后一个折线点与X轴垂线的交点
                        shadow.lineTo(startX + (pointCount - 1) * mXStep, mYPoint[0])
                        // 第一个折线点与X轴垂线的交点
                        shadow.lineTo(startX, mYPoint[0])
                        // 5.空间闭合,确定绘制阴影的区域
                        shadow.close()
                        val gradient = LinearGradient(
                                0f, mYPoint[0] - (model.max - minY) * coe,
                                0f, mYPoint[0],
                                model.shadowStartColor, model.shadowEndColor,
                                Shader.TileMode.CLAMP)
                        mLinePaint.style = Paint.Style.FILL
                        mLinePaint.shader = gradient
                        // 6.绘制阴影
                        canvas?.drawPath(shadow, mLinePaint)
                        mLinePaint.shader = null
                    }
                }
            }
        }
    }


}