package com.hexin.android.component.hangqing.hangqingfunds

import android.app.Dialog
import android.util.SparseArray
import android.view.KeyEvent
import android.view.LayoutInflater
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.hexin.android.theme.ThemeManager
import com.hexin.android.view.DrawableTextView
import com.hexin.android.view.DrawableTextView.LEFT_DRAWABLE
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.plat.android.R
import com.hexin.util.config.EQConstants
import com.hexin.util.config.SPConfig

class FundsWithIndexManager(private val hangQingFundsView: HangQingFundsView) : View.OnClickListener {
    private val szzsKey = 0
    private val szczKey = 1
    private val cybzKey = 2
    private val stockList = SparseArray<EQBasicStockInfo>()
    private var curSelectStockKey = EQConstants.INVALID
    init {
        val resources = hangQingFundsView.resources
        stockList.put(szzsKey, EQBasicStockInfo(resources.getString(R.string.szzs_name),
            EQConstants.STOCK_CODE_SHANGZHENG, EQConstants.MARKET_ID_SHANGZHENG))
        stockList.put(szczKey, EQBasicStockInfo(resources.getString(R.string.szcz_name),
            EQConstants.STOCK_CODE_SHENZHENG, EQConstants.MARKET_ID_SHENZHENG))
        stockList.put(cybzKey, EQBasicStockInfo(resources.getString(R.string.cybz_name),
            EQConstants.STOCK_CODE_CHANGYEBAN, EQConstants.MARKET_ID_CHANGYEBAN))
        curSelectStockKey = SPConfig.getIntSPValue(
            hangQingFundsView.context, SPConfig.SP_NAME_HQ_LIST, SPConfig.SP_KEY_ADD_INDEX_STOCK_CODE, szzsKey)
    }

    fun getSelectStock(): EQBasicStockInfo? {
        return stockList[curSelectStockKey]
    }

    fun showChooseStockDialog() {
        if (mDialog.isShowing) {
            return
        }
        initTheme()
        mDialog.show()
    }

    private fun initTheme() {
        val context = hangQingFundsView.context
        mChooseContainer.setBackgroundColor(ThemeManager.getColor(context, R.color.white_FFFFFF))
        // 这里是为了初始化mDialogBg和mBackIv以注册点击事件监听
        mDialogBg
        mBackIv
        val textColor = ThemeManager.getColor(context, R.color.gray_323232)
        mTitleTv.setTextColor(textColor)
        mNoIndexTv.setTextColor(textColor)
        mSZZSIndexTv.setTextColor(textColor)
        mSZCZIndexTv.setTextColor(textColor)
        mCYBZIndexTv.setTextColor(textColor)
        val uncheckDrawable = ThemeManager.getDrawableRes(context, R.drawable.icon_hot_stock_unchecked)
        mNoIndexTv.setDrawable(LEFT_DRAWABLE, uncheckDrawable)
        mNoIndexTv.setDrawable(LEFT_DRAWABLE, uncheckDrawable)
        mNoIndexTv.setDrawable(LEFT_DRAWABLE, uncheckDrawable)
        mNoIndexTv.setDrawable(LEFT_DRAWABLE, uncheckDrawable)
        for (split in mSplits) {
            split.setBackgroundColor(ThemeManager.getColor(context, R.color.gray_EEEEEE))
        }
        getCurSelectView().setDrawable(LEFT_DRAWABLE,
                ThemeManager.getDrawableRes(context, R.drawable.icon_hot_stock_checked))
    }

    override fun onClick(v: View?) {
        v?.let {
            if (it.id == R.id.choose_index_back || it.id == R.id.choose_index_bg) {
                dismissDialog()
            } else if (it is DrawableTextView){
                val clickKey = it.tag as Int
                if (clickKey != curSelectStockKey) {
                    val curSelectView = getCurSelectView()
                    curSelectView.setDrawable(LEFT_DRAWABLE,
                            ThemeManager.getDrawableRes(it.context, R.drawable.icon_hot_stock_unchecked))
                    it.setDrawable(LEFT_DRAWABLE,
                            ThemeManager.getDrawableRes(it.context, R.drawable.icon_hot_stock_checked))
                    curSelectStockKey = clickKey
                    hangQingFundsView.onIndexChanged(stockList[curSelectStockKey])
                    SPConfig.saveIntSPValue(hangQingFundsView.context,
                        SPConfig.SP_NAME_HQ_LIST, SPConfig.SP_KEY_ADD_INDEX_STOCK_CODE, clickKey
                    )
                }
            }
        }
    }

    private fun getCurSelectView(): DrawableTextView {
        return when (curSelectStockKey) {
            szzsKey -> mSZZSIndexTv
            szczKey -> mSZCZIndexTv
            cybzKey -> mCYBZIndexTv
            else -> mNoIndexTv
        }
    }

    fun dismissDialog() {
        if (!mDialog.isShowing) {
            return
        }
        mDialog.dismiss()
        hangQingFundsView.onDialogDismiss()
    }


    private val mDialog by lazy {
        val dialog = Dialog(hangQingFundsView.context, R.style.AddIndexDialog)
        dialog.setOnKeyListener { _, keyCode, event ->
            if (event.action == KeyEvent.ACTION_DOWN && keyCode == KeyEvent.KEYCODE_BACK) {
                dismissDialog()
                true
            } else {
                false
            }
        }
        dialog.setContentView(mContentView)
        dialog
    }
    private val mContentView by lazy { LayoutInflater.from(hangQingFundsView.context).
        inflate(R.layout.dialog_funds_choose_index, hangQingFundsView, false) }
    private val mBackIv by lazy {
        val view = mContentView.findViewById<ImageView>(R.id.choose_index_back)
        view.setOnClickListener(this)
        view
    }
    private val mTitleTv by lazy { mContentView.findViewById<TextView>(R.id.choose_index_title) }
    private val mNoIndexTv by lazy {
        val view = mContentView.findViewById<DrawableTextView>(R.id.choose_index_no_choose)
        view.tag = EQConstants.INVALID
        view.setOnClickListener(this)
        view
    }
    private val mSZZSIndexTv by lazy {
        val view = mContentView.findViewById<DrawableTextView>(R.id.choose_index_szzs)
        view.tag = szzsKey
        view.setOnClickListener(this)
        view
    }
    private val mSZCZIndexTv by lazy {
        val view = mContentView.findViewById<DrawableTextView>(R.id.choose_index_szcz)
        view.tag = szczKey
        view.setOnClickListener(this)
        view
    }
    private val mCYBZIndexTv by lazy {
        val view = mContentView.findViewById<DrawableTextView>(R.id.choose_index_cybz)
        view.tag = cybzKey
        view.setOnClickListener(this)
        view
    }

    private val mDialogBg by lazy {
        val view = mContentView.findViewById<View>(R.id.choose_index_bg)
        view.setOnClickListener(this)
        view
    }

    private val mSplits by lazy {
        arrayOf<View>(mContentView.findViewById(R.id.choose_index_split1),
            mContentView.findViewById(R.id.choose_index_split2),
            mContentView.findViewById(R.id.choose_index_split3),
            mContentView.findViewById(R.id.choose_index_split4))
    }

    private val mChooseContainer by lazy { mContentView.findViewById<View>(R.id.choose_index_container) }

}