package com.hexin.android.component.hangqing.hushen

import android.content.Context
import androidx.recyclerview.widget.RecyclerView
import android.util.AttributeSet
import android.view.MotionEvent

/**
 * 解决热门指数与ViewPager的滑动冲突，参考[com.hexin.android.component.RMZSLayout]
 */
class HotBlockRecyclerView(context: Context, attrs: AttributeSet?) : RecyclerView(context, attrs) {
    //保存ACTION_DOWN之后的坐标，用于解决滑动冲突
    private var lastX: Int = 0
    private var lastY: Int = 0

    override fun dispatchTouchEvent(ev: MotionEvent): Boolean {
        val x = ev.x.toInt()
        val y = ev.y.toInt()
        val action = ev.action
        when (action) {
            MotionEvent.ACTION_DOWN -> {
                //强制父布局不拦截事件
                parent.requestDisallowInterceptTouchEvent(true)
                lastX = x
                lastY = y
            }
            MotionEvent.ACTION_MOVE -> {
                val moveX = x - lastX
                val moveY = y - lastY
                //如果为上下滑动，给父布局处理，左右滑动，给自己处理
                if (Math.abs(moveX) < Math.abs(moveY)) {
                    parent.requestDisallowInterceptTouchEvent(false)
                }
            }
        }
        return super.dispatchTouchEvent(ev)
    }
}