package com.hexin.android.component.hangqing.hushen

import android.content.Context
import android.text.TextUtils
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.Space
import android.widget.TextView
import com.hexin.android.component.hangqing.hangqingindex.TradeStateManager
import com.hexin.android.component.hq.IHqItem
import com.hexin.android.theme.ThemeManager
import com.hexin.android.view.HXToast
import com.hexin.android.view.ShadowDrawable
import com.hexin.android.view.TagTextViewHelper
import com.hexin.middleware.MiddlewareProxy
import com.hexin.plat.android.R
import com.hexin.util.HexinUtils
import com.hexin.util.config.EQConstants

@Suppress("UNCHECKED_CAST")
/**
 * 沪深行情首页股票排行模块
 */
class StockSortListComponent(context: Context?, attrs: AttributeSet?) : LinearLayout(context, attrs), IHqItem {
    private val presenter by lazy { StockSortListPresenter(this) }
    private val divider by lazy { findViewById<View>(this, R.id.stock_list_divider) }
    private val vTop by lazy { findViewById<View>(this, R.id.v_top) }
    private val titleText by lazy { findViewById<TextView>(this,R.id.hushen_firstpage_title_text) }
    private val titleImage by lazy { findViewById<ImageView>(this,R.id.iv_forward) }
    private val tabSelector by lazy {
        TabSelector(arrayOf(findViewById<TextView>(this,R.id.price_rise_tab),
                findViewById<TextView>(this,R.id.price_fall_tab),
                findViewById<TextView>(this,R.id.rapid_price_rise_tab),
                findViewById<TextView>(this,R.id.volume_tab),
                findViewById<TextView>(this,R.id.net_volume_tab),
                findViewById<TextView>(this,R.id.volume_relative_ratio_tab),
                findViewById<TextView>(this,R.id.turnover_rate_tab),
                findViewById<TextView>(this,R.id.net_inflow_tab)), presenter)
    }
    private val currentPriceTitle by lazy { findViewById<TextView>(this, R.id.current_price_title) }
    private val variableTitle by lazy { findViewById<TextView>(this, R.id.variable_sort_title) }
    private val blockTitle by lazy { findViewById<TextView>(this, R.id.block_title) }
    private val recyclerView by lazy { findViewById<RecyclerView>(this, R.id.stock_list_recycle_view) }
    private val moreLayout by lazy { findViewById<View>(this, R.id.stock_list_more_layout) }
    private val moreText by lazy { findViewById<TextView>(this, R.id.hushen_firstpage_more_text) }
    private val emptyText by lazy { findViewById<TextView>(this, R.id.stock_list_empty_text) }
    private val analysisText: TextView by lazy {findViewById<TextView>(R.id.fold_textview)}
    private val analysisIcon: ImageView by lazy {findViewById<ImageView>(R.id.fold_icon)}
    protected val analysisContainer: LinearLayout by lazy {findViewById<LinearLayout>(R.id.fold_container)}
    protected var mIsAnalysisExpand = false
    private val bgShadowDrawable by lazy { ShadowDrawable(resources.getDimensionPixelOffset(R.dimen.dp_4), resources.getDimensionPixelOffset(R.dimen.dp_4)) }


    override fun onFinishInflate() {
        super.onFinishInflate()

        analysisContainer.setOnClickListener {
            mIsAnalysisExpand = !mIsAnalysisExpand
            val runtimeDataManager = MiddlewareProxy.getmRuntimeDataManager();
            runtimeDataManager?.setIsShowZTAnalysis(mIsAnalysisExpand)
            setAnalysisText()
        }
        initAnalysis()
        recyclerView.apply {
            adapter = SingleStockAdapter()
            layoutManager = LinearLayoutManager(context, LinearLayoutManager.VERTICAL, false)
            setHasFixedSize(true)
        }
        moreLayout.setOnClickListener {
            presenter.jumpToWholeListPage()
        }
        titleText.setText(R.string.stock_sort_list)
        titleImage.setOnClickListener {
            presenter.jumpToWholeListPage()
        }
        moreText.text = resources.getString(R.string.read_all)
    }

    private fun initAnalysis() {
        if (presenter.currentSelectType == StockSortListPresenter.SelectType.PRICE_RISE) {
            analysisContainer.visibility = View.VISIBLE
        } else {
            analysisContainer.visibility = View.INVISIBLE
        }
    }

    private fun setAnalysisText() {
        analysisText.text = resources.getString(
                if (mIsAnalysisExpand) R.string.kcb_list_title_text_close else R.string.kcb_list_title_text_open)
        val drawableRes = if (mIsAnalysisExpand) R.drawable.analysis_expand else R.drawable.analysis_collapse
        analysisIcon.setImageResource(ThemeManager.getDrawableRes(context, drawableRes))
        if (recyclerView.adapter is SingleStockAdapter) {
            val adapter = recyclerView.adapter as SingleStockAdapter
            adapter.foldChange(
                    mIsAnalysisExpand &&
                            presenter.currentSelectType == StockSortListPresenter.SelectType.PRICE_RISE
            )
            adapter.notifyDataSetChanged()
        }

        if (mIsAnalysisExpand) {
            recyclerView.setBackgroundResource(R.drawable.transparent)
        } else {
            recyclerView.background = bgShadowDrawable
        }
    }

    override fun onForeground() {
        var isExpand = false
        val runtimeDataManager = MiddlewareProxy.getmRuntimeDataManager();
        if (runtimeDataManager != null) {
            isExpand = runtimeDataManager.isShowZTAnalysis
        }
        if (isExpand != mIsAnalysisExpand) {
            mIsAnalysisExpand = isExpand
        }
        presenter.addRequestToBuffer()
        presenter.requestAnalysisData()
        initTheme()
    }

    override fun initTheme() {
        setBackgroundColor(ThemeManager.getColor(context, R.color.background_hq_row))
        vTop.setBackgroundColor(ThemeManager.getColor(context, R.color.gray_F5F5F5))
        divider.setBackgroundColor(ThemeManager.getColor(context, R.color.gray_F5F5F5))
        val gray323232 = ThemeManager.getColor(context, R.color.gray_323232)
        titleText.setTextColor(gray323232)
        emptyText.setTextColor(gray323232)
        tabSelector.initAllTheme()
        val gray999999 = ThemeManager.getColor(context, R.color.gray_999999)
        currentPriceTitle.setTextColor(gray999999)
        variableTitle.setTextColor(gray999999)
        blockTitle.setTextColor(gray999999)
        moreText.setTextColor(gray999999)
        analysisText.setTextColor(ThemeManager.getColor(context, R.color.gray_666666))
        setAnalysisText()
        recyclerView.adapter?.notifyDataSetChanged()
        bgShadowDrawable.setColor(ThemeManager.getColor(context, R.color.shadow))
    }

    override fun onTimeStateChange(model: TradeStateManager.TradeStateModel) {
//        val state = model.tradeState
//        presenter.currentTradeState = state
//        if (state == TradeStateManager.TRADE_STATE_INIT) {
//            showEmptyView(context.getString(R.string.trade_state_init_tips))
//        } else {
            refreshRecyclerView()
//        }
    }

    fun showToast(msg: String) {
        HXToast.makeText(context, msg, HXToast.HXTOAST_SHORT).show()
    }

    fun showEmptyView(text: String) {
        emptyText.visibility = View.VISIBLE
        moreLayout.visibility = View.GONE
        recyclerView.visibility = View.GONE
        emptyText.text = text
    }

    fun refreshRecyclerView() {
        emptyText.visibility = View.GONE
        moreLayout.visibility = View.VISIBLE
        recyclerView.visibility = View.VISIBLE
        recyclerView.adapter?.notifyDataSetChanged()
    }

    open fun setAnalysisVisible(isVisible: Boolean) {
        analysisContainer.visibility = if (isVisible) View.VISIBLE else View.INVISIBLE
        analysisContainer.isClickable = isVisible
        if (recyclerView.adapter is SingleStockAdapter) {
            val adapter = recyclerView.adapter as SingleStockAdapter
            adapter.foldChange(isVisible && mIsAnalysisExpand)
        }
    }

    /**
     * 根据选中的类型展示不同的排序标题
     */
    fun updateSortTitle(selectType: StockSortListPresenter.SelectType) {
        when (selectType) {
            StockSortListPresenter.SelectType.PRICE_RISE,
            StockSortListPresenter.SelectType.PRICE_FALL -> variableTitle.text =
                    context.getString(R.string.hangqing_bankuai_zhangfu)
            StockSortListPresenter.SelectType.RISE_SPEED -> variableTitle.text =
                    context.getString(R.string.hangqing_bankuai_zhangsu)
            StockSortListPresenter.SelectType.TRANSACTION -> variableTitle.text =
                    context.getString(R.string.wtyk_opt_money)
            StockSortListPresenter.SelectType.NET_VOLUME -> variableTitle.text =
                    context.getString(R.string.dadanjingliang)
            StockSortListPresenter.SelectType.VOLUME_RELATIVE_RATIO -> variableTitle.text =
                    context.getString(R.string.liangbi)
            StockSortListPresenter.SelectType.TURNOVER_RATE -> variableTitle.text =
                    context.getString(R.string.huanshou)
            StockSortListPresenter.SelectType.NET_INFLOW -> variableTitle.text =
                    context.getString(R.string.main_net_inflow)
        }
    }

    fun <T : View> findViewById(view: View, id: Int): T {
        return view.findViewById(id) as T
    }

    fun setPageType(type: StockSortListPresenter.SelectModelType) {
        presenter.onModelTabChange(type, false)
    }

    override fun onRemove() {
        presenter.releaseData()
    }

    override fun getTopView(): ImageView? = null

    override fun getItemView(context: Context?): View = this

    override fun getDivide(): View = divider

    override fun getName(): String = titleText.text.toString()

    override fun getKey(): String = EQConstants.TYPE_STOCK_RANK_VIEW

    override fun onBackground() {}

    override fun onPositionChange(position: Int) {}

    inner class SingleStockAdapter : RecyclerView.Adapter<BaseStockViewHolder>() {

        private val viewTypeCollapse = 0
        private val viewTypeExpand = 1

        var itemFoldStates : Boolean = false //是否展开分析数据

        fun foldChange(open : Boolean) {
            itemFoldStates = open
        }

        override fun getItemViewType(position: Int): Int {
            return if (itemFoldStates)
                viewTypeExpand else viewTypeCollapse
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseStockViewHolder {
            return if (viewType == viewTypeCollapse) {
                SingleStockViewHolder(LayoutInflater.from(context).inflate(
                        R.layout.hushen_firstpage_stock_list_item, parent, false))
            } else {
                ExpandStockViewHolder(LayoutInflater.from(context).inflate(
                        R.layout.hushen_firstpage_stock_list_expand_item, parent, false))
            }
        }

        override fun onBindViewHolder(holder: BaseStockViewHolder, position: Int) {
            val stockList = presenter.stocks
            if (position < stockList.size) {
                holder.apply {
                    val singleStockData = stockList[position]
                    val color323232 = ThemeManager.getColor(context, R.color.gray_323232)
                    nameView.text = transferToShowText(singleStockData.name)
                    nameView.setTextColor(color323232)
                    codeView.text = transferToShowText(singleStockData.code)
                    codeView.setTextColor(ThemeManager.getColor(context, R.color.gray_999999))
                    TagTextViewHelper.buildTagTextView(context, logoView, singleStockData.logoType)
                    currentPriceView.text = transferToShowText(singleStockData.currentPrice)
                    currentPriceView.setTextColor(HexinUtils.getTransformedColor(
                            singleStockData.currentPriceColor, context))
                    belongedBlockView.text = transferToShowText(singleStockData.belongedBlock)
                    belongedBlockView.setTextColor(color323232)
                    variableSortView.text = singleStockData.variableSortValue
                    variableSortView.setTextColor(HexinUtils.getTransformedColor(
                            singleStockData.variableSortColor, context))
                    itemView.setOnClickListener {
                        presenter.jumpToSingleBlockPage(position)
                    }
                    setDataForSingleStockView(holder, singleStockData)
                    if (this is ExpandStockViewHolder) {
                        setData(position != 0, singleStockData)
                    }
                }
            }
        }

        private fun setDataForSingleStockView(holder: BaseStockViewHolder,
                                              singleStockData: StockSortListPresenter.SingleStockInfo) {
            if (holder !is SingleStockViewHolder) {
                return
            }
            holder.apply {
                dividerView.setBackgroundColor(ThemeManager.getColor(context, R.color.gray_EEEEEE))
                //analysis layout 原科创板涨停分析布局
                if (!TextUtils.isEmpty(singleStockData.analysisDesc) && itemFoldStates) {
                    analysisContainer.visibility = View.VISIBLE
                    analysisDesc.text = singleStockData.analysisDesc
                    analysisDesc.setTextColor(ThemeManager.getColor(context, R.color.gray_666666))
                    singleStockData.releativeBlock?.let {
                        analysisRelativeBlock.removeAllViews()
                        it.forEach { item ->
                            val textView = TextView(context)
                            textView.setTextColor(ThemeManager.getColor(context, R.color.red_E93030_FD4332))
                            textView.text = item
                            textView.setBackgroundResource(
                                    ThemeManager.getDrawableRes(
                                            context,
                                            R.drawable.bg_block
                                    )
                            )
                            val lp = LinearLayout.LayoutParams(
                                    ViewGroup.LayoutParams.WRAP_CONTENT,
                                    ViewGroup.LayoutParams.WRAP_CONTENT
                            )
                            lp.rightMargin = context.resources.getDimensionPixelSize(R.dimen.dp_4)
                            analysisRelativeBlock.addView(textView, lp)
                        }
                    }
                } else {
                    analysisContainer.visibility = View.GONE
                }
            }
        }

        override fun getItemCount(): Int = Math.min(presenter.stocks.size, StockSortListPresenter.reqCount())
    }

    class SingleStockViewHolder(itemView: View) : BaseStockViewHolder(itemView) {
        val dividerView: View by lazy { itemView.findViewById<View>(R.id.stock_list_divider) }

        //analysis layout
        val analysisContainer : ViewGroup by lazy { itemView.findViewById<ViewGroup>(R.id.stock_list_analysis) }
        val analysisDesc : TextView by lazy {itemView.findViewById<TextView>(R.id.stock_list_analysis_desc)}
        val analysisRelativeBlock : LinearLayout by lazy {itemView.findViewById<LinearLayout>(R.id.stock_list_analysis_block)}
    }

    open class BaseStockViewHolder(itemView: View): RecyclerView.ViewHolder(itemView) {
        val nameView: TextView by lazy { itemView.findViewById<TextView>(R.id.stock_list_name) }
        val codeView: TextView by lazy { itemView.findViewById<TextView>(R.id.stock_list_code) }
        val logoView: TextView by lazy { itemView.findViewById<TextView>(R.id.stock_list_market_logo) }
        val currentPriceView: TextView by lazy { itemView.findViewById<TextView>(R.id.stock_list_current_price) }
        val belongedBlockView: TextView by lazy { itemView.findViewById<TextView>(R.id.stock_list_belonged_block) }
        val variableSortView: TextView by lazy { itemView.findViewById<TextView>(R.id.stock_list_variable_sort) }
    }

    inner class ExpandStockViewHolder(itemView: View): BaseStockViewHolder(itemView) {
        private val ztReasonTv by lazy {itemView.findViewById<TextView>(R.id.stock_list_zt_reason)}
        private val space: Space by lazy { itemView.findViewById<Space>(R.id.stock_list_divider) }
        private val stockTags by lazy {arrayOf<TextView>(itemView.findViewById(R.id.stock_list_tag1),
                itemView.findViewById(R.id.stock_list_tag2),
                itemView.findViewById(R.id.stock_list_tag3))}
        private val shadowDrawable by lazy { ShadowDrawable(resources.getDimensionPixelOffset(R.dimen.dp_4), resources.getDimensionPixelOffset(R.dimen.dp_4)) }
        init {
            itemView.findViewById<View>(R.id.stock_list_item_container).setBackgroundDrawable(shadowDrawable)
        }

        private fun hideAllTag() {
            for (tag in stockTags) {
                tag.visibility = View.GONE
            }
        }

        fun setData(isShowSpace: Boolean, stockModel: StockSortListPresenter.SingleStockInfo) {
            shadowDrawable.setColor(ThemeManager.getColor(context, R.color.shadow))
            space.visibility = if (isShowSpace) View.VISIBLE else View.GONE
            val reasonModel = presenter.mZTReasonMap[stockModel.code]
            if (reasonModel == null) {
                ztReasonTv.visibility = View.GONE
                hideAllTag()
            } else {
                ztReasonTv.visibility = View.VISIBLE
                ztReasonTv.text = reasonModel.ztReason
                ztReasonTv.setTextColor(ThemeManager.getColor(context, R.color.gray_666666))
                if (reasonModel.tags != null && !reasonModel.tags.isEmpty()) {
                    for ((index, tag) in reasonModel.tags.withIndex()) {
                        stockTags[index].visibility = View.VISIBLE
                        stockTags[index].text = tag
                        stockTags[index].setTextColor(ThemeManager.getColor(context, R.color.red_E93030))
                        stockTags[index].setBackgroundResource(ThemeManager.getDrawableRes(context, R.drawable.stock_tag_bg))
                    }
                    for (i in reasonModel.tags.size until stockTags.size) {
                        stockTags[i].visibility = View.INVISIBLE
                    }
                } else {
                    hideAllTag()
                }
            }
        }
    }

    companion object {
        private const val ITEM_VIEW_TYPE = 1
        private const val MORE_VIEW_TYPE = 2

        /**
         * 将空字符串或者“null”转换为“--”，处理返回空数据场景
         */
        fun transferToShowText(text: String): String {
            val trimText = text.trim()
            return if (trimText.isBlank() || trimText.equals(EQConstants.INVALID_STR_NULL, true)) {
                EQConstants.INVALID_STR_VALUE_DOUBLELINE
            } else {
                text
            }
        }
    }
}