package com.hexin.android.component.hangqing.hushen

import android.graphics.Color
import android.text.TextUtils
import com.hexin.android.component.hangqing.HQDataConstant
import com.hexin.android.component.hangqing.HangQingMarketTable
import com.hexin.android.service.UnifyUrlManager
import com.hexin.android.ui.NetWorkClinet
import com.hexin.android.ui.ScreenFoldConfigure
import com.hexin.android.view.TagTextViewHelper
import com.hexin.app.event.EQEvent
import com.hexin.app.event.action.EQGotoFrameAction
import com.hexin.app.event.action.EQGotoPageNaviAction
import com.hexin.app.event.action.EQGotoUnknownFrameAction
import com.hexin.app.event.param.EQGotoParam
import com.hexin.app.event.param.EQParam
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.app.event.struct.TitleLabelListStruct
import com.hexin.middleware.*
import com.hexin.middleware.data.StuffBaseStruct
import com.hexin.middleware.data.mobile.StuffTableStruct
import com.hexin.middleware.http.IRequest
import com.hexin.plat.android.R
import com.hexin.util.HexinCBASUtil
import com.hexin.util.HexinThreadPool
import com.hexin.util.HexinUtils
import com.hexin.util.Log
import com.hexin.util.config.EQConstants
import org.json.JSONException
import org.json.JSONObject

/**
 * 沪深首页股票排行业务处理与数据请求
 */
class StockSortListPresenter(private val view: StockSortListComponent) : NetWorkClinet, OnTabChangeListener {
    var currentSelectModelType = SelectModelType.TYPE_HUSHEN //当前选中模块tab
    var currentSelectType = SelectType.PRICE_RISE //当前选中tab
    var stocks: MutableList<SingleStockInfo> = mutableListOf()
    val mZTReasonMap = HashMap<String?, ZTAnalysisModel>()

    companion object {
        private val CBAS_PREFIX = "gpph"
        private val CBAS_TABS = arrayOf("zhangfu", "diefu", "zhangsu", "chengjiaoe", "ddjl", "liangbi", "huanshou", "zhulijlr")
        private val BLOCK_SORT_ID = 36072

        val MAX_ROW_COUNT = 8
        private const val TAG = "StockSortListPresenter"
        private val MAX_ROW_COUNT_FOLD_SCREEN = 20

        fun reqCount(): Int {
            return if (ScreenFoldConfigure.isExpand() && ScreenFoldConfigure.isHWFoldScreen()) {
                MAX_ROW_COUNT_FOLD_SCREEN
            } else {
                MAX_ROW_COUNT
            }
        }

        /**
         * 从字符串数组获取对应index值，做了空处理
         */
        fun getValueAt(index: Int, datas: Array<String?>?): String {
            if (datas != null && datas.size > index) {
                val value = datas[index]
                if (value != null) {
                    return value
                }
            }
            return ""
        }

        /**
         * 从Int颜色数组获取对应index值，做了空处理
         */
        fun getColorAt(index: Int, datas: IntArray?): Int {
            if (datas != null && datas.size > index) {
                val value = datas[index]
                return value
            }
            return Color.BLACK
        }

        /**
         * 跳转对应个股分时，实现了标题栏设置以及埋点发送
         */
        fun jumpToSingleStockPage(stocks: List<SimpleStockInfo>, chosenIndex: Int, cbasObject: String, frameId: Int) {
            if (chosenIndex in 0 until stocks.size) {
                val stock = stocks[chosenIndex]
                addInfoToTitleBar(stocks, chosenIndex)
                val gotoFrameAction = EQGotoUnknownFrameAction.createEQAction(frameId, stock.market)
                val gotoParam = EQGotoParam(EQParam.PARAM_TYPE_STOCKINFO,
                        EQBasicStockInfo(stock.name, stock.code, stock.market))
                gotoParam.setUsedForAll()
                gotoFrameAction.param = gotoParam
                HexinCBASUtil.sendStandardJumpPageCbas(cbasObject,
                        gotoFrameAction.gotoFrameId, null, true, stock.code)
                MiddlewareProxy.executorAction(gotoFrameAction)
            }
        }

        /**
         * 将股票列表信息添加到标题栏对应的数据结构中，保证可以进行股票切换
         */
        private fun addInfoToTitleBar(stocks: List<SimpleStockInfo>, currentIndex: Int) {
            val titleLabelListStruct = TitleLabelListStruct()
            val stockNameList = com.hexin.util.data.List()
            val stockCodeList = com.hexin.util.data.List()
            val marketIdList = com.hexin.util.data.List()
            for (i in 0 until stocks.size) {
                stockNameList.add(stocks[i].name)
                stockCodeList.add(stocks[i].code)
                marketIdList.add(stocks[i].market)
            }
            titleLabelListStruct.stockListIndex = currentIndex
            titleLabelListStruct.stockNameList = stockNameList
            titleLabelListStruct.stockCodeList = stockCodeList
            titleLabelListStruct.stockMarketIdList = marketIdList
            titleLabelListStruct.isSameMarket = HexinUtils.isAllSameMarketIdInList(marketIdList)
            MiddlewareProxy.saveTitleLabelListStruct(titleLabelListStruct)
        }
    }

    override fun onTabChanged(select: Int) {
        when (select) {
            0 -> currentSelectType = SelectType.PRICE_RISE
            1 -> currentSelectType = SelectType.PRICE_FALL
            2 -> currentSelectType = SelectType.RISE_SPEED
            3 -> currentSelectType = SelectType.TRANSACTION
            4 -> currentSelectType = SelectType.NET_VOLUME
            5 -> currentSelectType = SelectType.VOLUME_RELATIVE_RATIO
            6 -> currentSelectType = SelectType.TURNOVER_RATE
            7 -> currentSelectType = SelectType.NET_INFLOW
        }
        view.updateSortTitle(currentSelectType)
        view.setAnalysisVisible(currentSelectType == SelectType.PRICE_RISE)
        addRequestToBuffer()
        MiddlewareProxy.requestFlush(true)
    }

    /**
     * 沪深和科创版切换
     */
    fun onModelTabChange(select : SelectModelType, isNeedRequest: Boolean) {
        currentSelectModelType = select
        view.setAnalysisVisible(currentSelectType == SelectType.PRICE_RISE)
        view.updateSortTitle(currentSelectType)
        if (isNeedRequest) {
            addRequestToBuffer()
            MiddlewareProxy.requestFlush(true)
        }
    }

    fun addRequestToBuffer() {
        //默认降序请求前5条
        var requestText = """
            startrow=0
            rowcount=8
            sortorder=${getRequestSortOrder()}
            sortid=${getRequestSortId()}
            marketid=${getMarketId()}
        """.trimIndent()


        MiddlewareProxy.addRequestToBuffer(2312,
                ProtocalDef.PAGEID_SC, QueueManagement.getInstanceId(this), requestText)
    }

    open fun requestAnalysisData() {
        HexinThreadPool.getThreadPool().execute {
            val url = UnifyUrlManager.getInstance().getUrlByResourceId(R.string.first_page_zhangting_url)
            val response = IRequest.post(url).execute()
            val result = parseAnalysisData(response)
            GlobalHandler.post {
                if (result != null && !result.isEmpty()) {
                    mZTReasonMap.clear()
                    // 这里已经保证了result中的每一个数据都是有效的,即stockCode不为空
                    for (model in result) {
                        mZTReasonMap[(model.stockCode)] = model
                    }
                    view.refreshRecyclerView()
                }
            }
        }
    }

    private fun parseAnalysisData(data: String?): List<ZTAnalysisModel>? {
        if (TextUtils.isEmpty(data)) {
            return null
        }
        val result = ArrayList<ZTAnalysisModel>()
        return try {
            val jsonObject = JSONObject(data)
            val errorNo = jsonObject.optString(ZTAnalysisModel.JSON_KEY_ERROR_NO)
            if (ZTAnalysisModel.JSON_DATA_RIGHT_FLAG != errorNo) {
                // 数据无效
                val errorInfo = jsonObject.optString(ZTAnalysisModel.JSON_KEY_ERROR_INFO)
                GlobalHandler.post {
                    view.showToast(errorInfo)
                }
                return null
            }
            val jsonArray = jsonObject.optJSONArray(ZTAnalysisModel.JSON_KEY_STOCKS)
            var analysisModel: ZTAnalysisModel?
            for (i in 0 until jsonArray.length()) {
                analysisModel = parseAnalysisModel(jsonArray.optJSONObject(i))
                if (analysisModel?.isValid() == true) {
                    result.add(analysisModel)
                }
            }
            result
        } catch (jsonException: JSONException) {
            Log.printStackTrace(jsonException)
            Log.e(TAG, "parse error")
            null
        }
    }

    private fun parseAnalysisModel(jsonObject: JSONObject): ZTAnalysisModel? {
        try {
            val tagArray = jsonObject.optJSONArray(ZTAnalysisModel.JSON_KEY_STOCK_LABELS)
            val tagList: List<String>?
            if (tagArray != null) {
                tagList = ArrayList()
                for (i in 0 until Math.min(ZTAnalysisModel.MAX_SHOW_TAGS, tagArray.length())) {
                    tagList.add(tagArray.getJSONObject(i).getString(ZTAnalysisModel.JSON_KEY_LABEL_NAME))
                }
            } else {
                tagList = null
            }
            val description = jsonObject.optString(ZTAnalysisModel.JSON_KEY_DESCRIPTION)
            return ZTAnalysisModel(
                    jsonObject.optString(ZTAnalysisModel.JSON_KEY_STOCK_CODE),
                    jsonObject.optString(ZTAnalysisModel.JSON_KEY_STOCK_NAME),
                    description?.replace("\n", EQConstants.DESCRIPTION_EMPTY_STRING), tagList
            )
        } catch (jsonException: JSONException) {
            Log.printStackTrace(jsonException)
            Log.e(TAG, "parse error")
            return null
        }
    }

    private fun getMarketId() : String{
        if (currentSelectModelType == SelectModelType.TYPE_KCB) {
            return "65"
        }
        return "0"
    }

    private fun getRequestSortOrder(): Int {
        return if (currentSelectType == SelectType.PRICE_FALL) {
            EQConstants.DEFAULT_SORT_ORDER_UP
        } else {
            EQConstants.DEFAULT_SORT_ORDER_DOWN
        }
    }

    private fun getRequestSortId(): Int {
        return when (currentSelectType) {
            SelectType.PRICE_RISE -> ProtocalDef.HANGQING_AG_ZHANGFU
            SelectType.PRICE_FALL -> ProtocalDef.HANGQING_AG_ZHANGFU
            SelectType.RISE_SPEED -> ProtocalDef.SM_HQ_PERRISE
            SelectType.TRANSACTION -> ProtocalDef.HANGQING_STOCK_JINE
            SelectType.NET_VOLUME -> ProtocalDef.HANGQING_STOCK_DDJL
            SelectType.VOLUME_RELATIVE_RATIO -> ProtocalDef.SM_HQ_LIANGBI
            SelectType.TURNOVER_RATE -> ProtocalDef.SM_HQ_HUANSHOU
            SelectType.NET_INFLOW -> ProtocalDef.HANGQING_STOCK_ZLJLR
        }
    }

    override fun receive(struct: StuffBaseStruct?) {
        if (struct is StuffTableStruct) {
            val parsedStocks = parseBlockList(struct)
            GlobalHandler.post {
                if (parsedStocks.isNotEmpty()) {
                    stocks = parsedStocks
//                    if (currentTradeState != TradeStateManager.TRADE_STATE_INIT) {
                        view.refreshRecyclerView()
//                    }
                }
            }
        }
    }

    private fun parseBlockList(struct: StuffTableStruct): MutableList<SingleStockInfo> {
        val blockList = mutableListOf<SingleStockInfo>()
        val row = struct.row //主站返回的板块数目
        val codes = struct.getData(ProtocalDef.HANGQING_STOCK_CODE_4)
        val names = struct.getData(ProtocalDef.HANGQING_STOCK_NAME)
        val markets = struct.getData(ProtocalDef.HANGQING_STOCK_MARKETID)
        val currentPrices = struct.getData(ProtocalDef.HANGQING_STOCK_PRICE)
        val priceColors = struct.getDataColor(ProtocalDef.HANGQING_STOCK_PRICE)
        val belongedBlocks = struct.getData(BLOCK_SORT_ID) //所属板块
        val sortId = getRequestSortId()
        val specialValues = struct.getData(sortId)
        // 只处理成交额的数据，券商要求
        if (sortId == ProtocalDef.HANGQING_STOCK_JINE && specialValues != null) {
            transUnitString(specialValues)
        }
        val specialColors = struct.getDataColor(sortId)
        val logoTypes = struct.getData(HQDataConstant.HQ_STOCKFLAG)
        for (i in 0 until row) {
            val singleStockInfo = SingleStockInfo(getValueAt(i, codes),
                    getValueAt(i, names), getValueAt(i, markets))
            singleStockInfo.currentPrice = getValueAt(i, currentPrices)
            singleStockInfo.currentPriceColor = getColorAt(i, priceColors)
            singleStockInfo.belongedBlock = getValueAt(i, belongedBlocks)
            singleStockInfo.variableSortValue = getValueAt(i, specialValues)
            singleStockInfo.variableSortColor = getColorAt(i, specialColors)
            val logoTypeString = getValueAt(i, logoTypes)
            singleStockInfo.logoType = TagTextViewHelper.getStockTypeLogo(logoTypeString, singleStockInfo.market, singleStockInfo.code)
            blockList.add(singleStockInfo)
        }
        return blockList
    }

    // 只处理数据单位为万万时，改为亿
    fun transUnitString(valueArr : Array<String>) {
        // 防止越界等异常
        try {
            for (i in valueArr.indices) {
                val value = valueArr[i]
                if (!TextUtils.isEmpty(value) && value.contains(EQConstants.UNIT_TEN_THOUSAND)) {
                    val valueDoubleStr = value.replace(EQConstants.UNIT_TEN_THOUSAND, "")
                    val valueDouble = valueDoubleStr.toDouble() * 10000
                    valueArr[i] = HexinUtils.formatNumberStr(valueDouble.toString())
                }
            }
        } catch (e : Exception) {
            e.printStackTrace()
        }

    }

    fun jumpToSingleBlockPage(chosenIndex: Int) {

        jumpToSingleStockPage(stocks, chosenIndex, "", ProtocalDef.FRAMEID_STOCK_GG_PRICE)
    }

    fun jumpToWholeListPage() {
        val detailAction = getDetailAction()
        MiddlewareProxy.executorAction(detailAction)
    }

    private fun getDetailAction(): EQGotoFrameAction {
        val rdm = MiddlewareProxy.getmRuntimeDataManager()
        if (rdm != null) {
            rdm.hushenMarketSortId = mapToOldOrderIndex()
        }
        val gotoFrameAction = EQGotoFrameAction(EQEvent.EVENT_TYPE_CLICK, ProtocalDef.FRAMEID_SC_ROBOT)
        val ctrlId = if (currentSelectModelType == SelectModelType.TYPE_KCB)
            ProtocalDef.CTRLID_HANGQING_KCB_TABLE else ProtocalDef.CTRLID_HANGQING_HSAG_TABLE
        val gotoParam = EQGotoParam(EQParam.PARAM_TYPE_HQ_CTRLID, ctrlId)
        gotoParam.setUsedForAll()
        gotoFrameAction.param = gotoParam
        return gotoFrameAction
    }

    /**
     * 原来沪深首页点击更多时排序状态是通过[com.hexin.app.RuntimeDataManager.hushenMarketSortId]保存
     * 这里实现现有tab与排序状态的映射
     */
    private fun mapToOldOrderIndex(): Int {
        return when (currentSelectType) {
            SelectType.PRICE_FALL -> HangQingMarketTable.DIEFU
            SelectType.RISE_SPEED -> HangQingMarketTable.KSZF
            SelectType.TRANSACTION -> HangQingMarketTable.CJE
            SelectType.VOLUME_RELATIVE_RATIO -> HangQingMarketTable.LB
            SelectType.TURNOVER_RATE -> HangQingMarketTable.HSL
            SelectType.NET_VOLUME -> HangQingMarketTable.DDJL
            SelectType.NET_INFLOW -> HangQingMarketTable.ZLJLR
            else -> HangQingMarketTable.ZHANGFU
        }
    }

    fun releaseData() {
        QueueManagement.remove(this)
    }

    override fun request() {}

    open class SimpleStockInfo(open val code: String, open val name: String, open val market: String)



    data class BaseBlockData(override val code: String, override val name: String, override val market: String)
        : SimpleStockInfo(code, name, market) {
        var priceChange: String = ""
        var priceChangeColor: Int = Color.BLACK
        var inflow: String = ""
        var inflowColor: Int = Color.BLACK
    }

    data class SingleStockInfo(override val code: String, override val name: String, override val market: String)
        : SimpleStockInfo(code, name, market) {
        var currentPrice: String = ""
        var currentPriceColor: Int = Color.BLACK
        var variableSortValue: String = "" //根据选中的类型不同，字段的实际意义会有区别
        var variableSortColor: Int = Color.BLACK
        var belongedBlock: String = ""
        var logoType: Int = Color.BLACK
        //analysis data
        var analysisDesc : String = ""
        var releativeBlock : ArrayList<String>? = null
    }

    enum class SelectModelType { TYPE_HUSHEN, TYPE_KCB }

    enum class SelectType { PRICE_RISE, PRICE_FALL, RISE_SPEED, TRANSACTION, NET_VOLUME, VOLUME_RELATIVE_RATIO, TURNOVER_RATE, NET_INFLOW }
}