package com.hexin.android.component.hangqing.hushen

import android.view.View
import android.widget.TextView
import com.hexin.android.theme.ThemeManager
import com.hexin.plat.android.R

/**
 * 处理多TextView类型tab的切换逻辑，主要处理选中tab和非选中tab的背景，将这部分逻辑与tab布局分开
 */
class TabSelector(private val textViews: Array<TextView>, private val tabChangeListener: OnTabChangeListener) : View.OnClickListener {
     var currentSelectIndex = 0
         set(value) {
             if (value >= 0 && value < textViews.size) {
                 field = value
                 initAllTheme()
             }
         }
    var clickable: Boolean = true
        set(value) {
            field = value
            for (textView in textViews) {
                textView.isClickable = value
            }
            initAllTheme()
        }
    init {
        textViews.forEach {
            it.setOnClickListener(this)
        }
    }

    override fun onClick(clickView: View?) {
        val size = textViews.size
        for (i in 0 until size) {
            val currentView = textViews[i]
            if (currentView == clickView && i != currentSelectIndex) { //点击与当前已经选中的视图不同
                if (currentSelectIndex in 0 until size) {
                    setUnSelectTheme(textViews[currentSelectIndex])
                }
                currentSelectIndex = i
                setSelectTheme(currentView)
                tabChangeListener.onTabChanged(i)
                break
            }
        }
    }

    private fun setUnSelectTheme(textView: TextView) {
        val context = textView.context
        if (!clickable) {
            textView.alpha = 0.5f
        } else {
            textView.alpha = 1.0f
        }
        textView.setBackgroundResource(ThemeManager.getDrawableRes(context, R.drawable.feed_hs_tab_bg))
        textView.setTextColor(ThemeManager.getColor(context, R.color.gray_666666))
    }

    private fun setSelectTheme(textView: TextView) {
        val context = textView.context
        textView.alpha = 1.0f
        textView.setBackgroundResource(ThemeManager.getDrawableRes(context, R.drawable.feed_hs_tab_select_bg))
        textView.setTextColor(ThemeManager.getColor(context, R.color.red_E93030))
    }

    fun initAllTheme() {
        val size = textViews.size
        for (i in 0 until size) {
            if (i == currentSelectIndex) {
                setSelectTheme(textViews[i])
            } else {
                setUnSelectTheme(textViews[i])
            }
        }
    }

}