package com.hexin.android.component.hq

import android.content.Context
import android.text.TextUtils
import android.util.AttributeSet
import android.widget.LinearLayout
import android.widget.TextView
import com.hexin.android.component.hangqing.DigitalTextView
import com.hexin.android.component.hangqing.selfcode.SelfCodeDpVoerLayUtil
import com.hexin.android.component.hangqing.selfcode.SelfDPInfoDataHolder
import com.hexin.android.component.hq.kcb.KCBZStockInfoManager
import com.hexin.android.theme.ThemeManager
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.plat.android.R
import com.hexin.util.HexinUtils
import com.hexin.util.Log
import com.hexin.util.StringUtils
import com.hexin.util.config.EQConstants

/**
 * 通用的分时指数条
 * Created by yuegang on 19-5-23.
 */
const val TAG = "FenshiValueBar"
open class FenshiValueBar(context: Context?, attrs: AttributeSet?) : LinearLayout(context, attrs), StockValueBarClient.DataCallback {


    private val mStockNameView: TextView by lazy { findViewById<TextView>(R.id.stockname_txt) }
    private val mXianjiaValueView: DigitalTextView by lazy { findViewById<DigitalTextView>(R.id.price) }
    private val mZhangfuValueView: DigitalTextView by lazy { findViewById<DigitalTextView>(R.id.zhangdie_value) }
    private val mZhangfuRateView: DigitalTextView by lazy { findViewById<DigitalTextView>(R.id.zhangdie_rate) }
    private val mShizhiValueView: DigitalTextView by lazy { findViewById<DigitalTextView>(R.id.shizhi_value) }
    var mNameFromServer : String = KCB_NAME_DEFAULT
    companion object {
        private const val SHIZHI_UNIT = "亿"
        const val KCB_NAME_DEFAULT = "--"
        const val INDEX_XIANJIA = 0
        const val INDEX_ZHANGFU = 1
        const val INDEX_ZHANGFU_RATE = 2
        const val INDEX_SHIZHI = 3
        const val INDEX_STOCKCODE = 4
        const val INDEX_STOCKNAME = 5
        const val COUNT_TEN_PADDING = 6
        const val DATAIDS_CODE_NAME_INDEX = 5
        const val DATAIDS_CODE_INDEX = 4
    }

    override fun callback(data: StockValueBarClient.DPDataModel?, ids: IntArray) {
        data?.let {
            val holderData = findAndTransferDataModel(data)
            holderData?.let {
                updateValue(it, getIds())
            } ?: defaultValue()
        } ?: defaultValue()
    }


    override fun onFinishInflate() {
        super.onFinishInflate()
    }

    protected fun updateValue(dataHolder : SelfDPInfoDataHolder , ids: IntArray) {
        if (getKCBZStock().isEmpty()) {
            Log.w(Log.AM_HQ_TABLE, "FenshiValueBar_updateValue() : return cause stocks is empty.")
            return
        }
        if (dataHolder.size != ids.size) {
            return
        }
        val serverName = dataHolder.getString(ids[INDEX_STOCKNAME])
        mNameFromServer = if (!TextUtils.isEmpty(serverName) && EQConstants.DEFAULT_STR != serverName) {
            serverName
        } else {
            KCB_NAME_DEFAULT
        }
        mStockNameView.text = mNameFromServer
        mStockNameView.setTextColor(ThemeManager.getColor(context, R.color.text_dark_color))

        mXianjiaValueView.text = StringUtils.format2Decimal(dataHolder.getString(getIds()[INDEX_XIANJIA]))
        mXianjiaValueView.setTextColor(HexinUtils.getTransformedColor(dataHolder.getColor(ids[INDEX_XIANJIA]), context))

        mZhangfuValueView.text = StringUtils.format2Decimal(dataHolder.getString(ids[INDEX_ZHANGFU]))
        mZhangfuValueView.setTextColor(HexinUtils.getTransformedColor(dataHolder.getColor(ids[INDEX_ZHANGFU]), context))

        mZhangfuRateView.text = StringUtils.format2Decimal(dataHolder.getString(ids[INDEX_ZHANGFU_RATE]))
        mZhangfuRateView.setTextColor(HexinUtils.getTransformedColor(dataHolder.getColor(ids[INDEX_ZHANGFU_RATE]), context))

        var shizhiValue = dataHolder.getString(ids[INDEX_SHIZHI])
        if (shizhiValue.endsWith(SHIZHI_UNIT)) {
            val tempShizhiValue = shizhiValue.replace(SHIZHI_UNIT, "")
            if (StringUtils.isNumerical(tempShizhiValue)) {
                val shizhiFloatValue = tempShizhiValue.toFloat()
                shizhiValue = StringUtils.format2Decimal(shizhiFloatValue).plus(SHIZHI_UNIT)
            }
        } else {
            if (StringUtils.isNumerical(shizhiValue)) {
                val shizhiFloatValue = shizhiValue.toFloat()
                shizhiValue = StringUtils.format2Decimal(shizhiFloatValue).plus(SHIZHI_UNIT)
            }
        }
        mShizhiValueView.text = shizhiValue
        mShizhiValueView.setTextColor(HexinUtils.getTransformedColor(dataHolder.getColor(ids[INDEX_SHIZHI]), context))
    }


    // 从当前数据模型中解析出当前显示股票的数据
    private fun findAndTransferDataModel(dataModel: StockValueBarClient.DPDataModel): SelfDPInfoDataHolder? {
        if (getKCBZStock().isEmpty()) {
            Log.w(Log.AM_HQ_TABLE, "FenshiValueBar_findAndTransferDataModel() : return null cause stocks is empty.")
            return null
        }
        val currentCode = getKCBZStock()[0].mStockCode
        if (!TextUtils.isEmpty(currentCode)) {
            val valuesCount = dataModel.size
            val dataCount = getIds().size
            val codes = dataModel.getvalues(getIds()[DATAIDS_CODE_INDEX])
            val currentCodeColIndex = findCurrentStockCodeIndex(codes, currentCode)
            if (currentCodeColIndex >= 0 && valuesCount == dataCount) {
                val dataHolder = SelfDPInfoDataHolder()
                for (i in 0 until dataCount) {
                    val dataId = getIds()[i]
                    val values = dataModel.getvalues(dataId)
                    val colors = dataModel.getColors(dataId)
                    if (values != null && colors != null && values.size == colors.size && values.size > currentCodeColIndex) {
                        dataHolder.putColor(dataId,  colors[currentCodeColIndex])
                        dataHolder.putString(dataId, values[currentCodeColIndex])
                    }
                }
                return dataHolder
            }
        }
        return null
    }

    private fun findCurrentStockCodeIndex(codes: Array<String>?, currentCode: String): Int {
        if (codes == null || codes.isEmpty() || TextUtils.isEmpty(currentCode)) {
            return -1
        }
        return codes.indexOf(currentCode)
    }

    private fun defaultValue() {
        val defaultColor = ThemeManager.getColor(context, R.color.text_dark_color)
        mXianjiaValueView.text = KCB_NAME_DEFAULT
        mXianjiaValueView.setTextColor(defaultColor)

        mZhangfuValueView.text = KCB_NAME_DEFAULT
        mZhangfuValueView.setTextColor(defaultColor)

        mZhangfuRateView.text = KCB_NAME_DEFAULT
        mZhangfuRateView.setTextColor(defaultColor)

        mShizhiValueView.text = KCB_NAME_DEFAULT
        mShizhiValueView.setTextColor(defaultColor)

        mStockNameView.text = KCB_NAME_DEFAULT
        mStockNameView.setTextColor(defaultColor)
    }

    private val mClient : StockValueBarClient = StockValueBarClient.newInst(getIds())

    protected fun  getIds(): IntArray {
        return SelfCodeDpVoerLayUtil.DATAIDS_DP_PRICE
    }

    private fun getKCBZStock(): List<EQBasicStockInfo> {
        return if(KCBZStockInfoManager.getInstance().kcbzStockInfo != null){
            mutableListOf(KCBZStockInfoManager.getInstance().kcbzStockInfo)
        } else {
            mutableListOf()
        }
    }

    fun onForeground() {
        mClient.reqStocks = getKCBZStock()
        mClient.mCallback = this
        mClient.request()

        initTheme()
    }

    fun initTheme() {
    }

    fun onBackground() {
        mClient.stopClient()
    }
}