package com.hexin.android.component.hq

import android.text.TextUtils
import com.hexin.android.component.hq.item.HSItemViewConfig.TYPE_BID_VIEW
import com.hexin.android.component.hq.item.HSItemViewConfig.TYPE_MARKET_STATUS_VIEW
import com.hexin.util.config.SPConfig

/**
 * 因为需求需要先显示涨跌停的引导，带涨跌停引导消失后再显示大盘异动处的引导，因此该类用来控制引导的显示顺序
 * 在模块onForeground时通过isNeedShowGuide判断是否需要显示引导，引导被点击后调用saveGuideHasShow记录引导已经显示
 * */
class HQGuideManager {
    companion object {
        const val GUIDE_BID = 1     // 集合竞价异动
        const val GUIDE_DP_BID = 1 shl 1// 大盘异动
        private val guideOrder = arrayOf(TYPE_MARKET_STATUS_VIEW, TYPE_BID_VIEW)
        fun isNeedShowGuide(itemType: String): Boolean {
            val guideProgress = SPConfig.getIntSPValue(SPConfig.SP_NAME_HQ_LIST,
                SPConfig.SP_KEY_HQ_GUIDE_PROGRESS, 0)
            if (guideProgress < 0 || guideProgress >= guideOrder.size) {
                return false
            }
            return TextUtils.equals(itemType, guideOrder[guideProgress])
        }

        fun saveGuideHasShow(itemType: String) {
            for (i in 0 until guideOrder.size) {
                if (TextUtils.equals(itemType, guideOrder[i])) {
                    SPConfig.saveIntSPValue(SPConfig.SP_NAME_HQ_LIST, SPConfig.SP_KEY_HQ_GUIDE_PROGRESS, i + 1)
                    return
                }
            }
        }

        fun saveBidGuideHasShow(guideType: Int) {
            var bidGuideProgress = SPConfig.getIntSPValue(SPConfig.SP_NAME_HQ_LIST, SPConfig.SP_KEY_HQ_BID_GUIDE_PROGRESS, 0)
            bidGuideProgress = bidGuideProgress or guideType
            if (bidGuideProgress == GUIDE_BID or GUIDE_DP_BID) {
                saveGuideHasShow(TYPE_BID_VIEW)
            }
            SPConfig.saveIntSPValue(SPConfig.SP_NAME_HQ_LIST, SPConfig.SP_KEY_HQ_BID_GUIDE_PROGRESS, bidGuideProgress)
        }

        /**
         * 调用前需要先调用isNeedShowGuide判断是否需要显示异动模块的引导
         * */
        fun isNeedShowBidGuide(guideType: Int): Boolean {
            val bidGuideProgress = SPConfig.getIntSPValue(SPConfig.SP_NAME_HQ_LIST, SPConfig.SP_KEY_HQ_BID_GUIDE_PROGRESS, 0)
            return guideType and bidGuideProgress == 0
        }
    }

}