package com.hexin.android.component.hq

import android.annotation.SuppressLint
import android.content.Context
import android.util.AttributeSet
import android.view.MotionEvent
import android.widget.ScrollView

/**
 * 可以监听滚动停止以及滚动到底部的ScrollView
 */
open class ScrollViewWithListener(context: Context?, attrs : AttributeSet) : ScrollView(context, attrs) {
    var listener: ScrollListener? = null
    var mOverScrollerListener : OnOverScrolledListener? = null
    var onScrollStopListener: OnScrollStopListener? = null
    private var mInitialScrollY = 0
    private val mScrollerTask = Runnable {
        if (scrollY == mInitialScrollY) {
            onScrollStopListener?.onScrollStop()
        } else {
            startScrollStopTask()
        }
    }

    fun startScrollStopTask() {
        if (listener != null) {
            mInitialScrollY = scrollY
            postDelayed(mScrollerTask, 100)
        }
    }

    @SuppressLint("ClickableViewAccessibility")
    override fun onTouchEvent(ev: MotionEvent?): Boolean {
        if (listener != null && (ev?.action == MotionEvent.ACTION_UP || ev?.action == MotionEvent.ACTION_CANCEL)) {
            startScrollStopTask() //这种轮询的判断是否停止的方式并不好,只在需要的时候使用
        }
        return super.onTouchEvent(ev)
    }

    override fun onScrollChanged(l: Int, t: Int, oldl: Int, oldt: Int) {
        super.onScrollChanged(l, t, oldl, oldt)
        listener?.onScrollChang(l, t, oldl, oldt)
    }

    override fun onOverScrolled(scrollX: Int, scrollY: Int, clampedX: Boolean, clampedY: Boolean) {
        super.onOverScrolled(scrollX, scrollY, clampedX, clampedY)
        mOverScrollerListener?.onOverScrolled(scrollX, scrollY, clampedX, clampedY)
    }

    interface ScrollListener {
        fun onScrollChang(l: Int, t: Int, oldl: Int, oldt: Int)
    }

    interface OnScrollStopListener {
        fun onScrollStop()
    }

    interface OnOverScrolledListener {
        fun onOverScrolled(scrollX : Int, scrollY : Int, clampedX : Boolean, clampedY : Boolean)
    }
}