package com.hexin.android.component.hq

import android.util.SparseArray
import com.hexin.android.component.SelfStockTab
import com.hexin.android.component.hangqing.selfcode.SelfCodeDpVoerLayUtil
import com.hexin.android.ui.NetWorkClinet
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.middleware.*
import com.hexin.middleware.data.StuffBaseStruct
import com.hexin.middleware.data.mobile.StuffTableStruct
import com.hexin.middleware.data.realdata.model.RTDataSubscriber
import com.hexin.util.Log
import com.hexin.util.config.EQConstants

/**
 * 各种指数条数据请求和分发
 * Created by yuegang on 19-5-22.
 */
open class StockValueBarClient constructor(ids : IntArray = SelfCodeDpVoerLayUtil.DATAIDS_DP_PRICE): NetWorkClinet {


    companion object {
        /**
         * create a new client instance with request ids, use default ids if the params ids is null.
         */
        fun newInst(ids : IntArray?) : StockValueBarClient {
            return ids?.let { StockValueBarClient(ids) } ?: StockValueBarClient()
        }
    }

    val TAG = "StockValueBarClient"

    val REQUEST_TEXT = "stocklist=%s\r\nmarketlist=%s\r\nrowcount=%s\r\nnewrealtime=1"

    var mCallback : DataCallback? = null
    private var mParseIntercept : DataParserInterception? = null
    private val mIds : IntArray = ids
    var reqStocks : List<EQBasicStockInfo> = mutableListOf()


    override fun request() {
        request(false)
    }

    fun request(flush : Boolean) {
        if (reqStocks.isEmpty()) {
            Log.w(Log.AM_HQ_TABLE, "${TAG}_request() : return cause stock list is empty.")
            return
        }
        val params = buildRequestParams()
        if (EQConstants.INVALID_STR_VALUE == params) {
            Log.w(Log.AM_HQ_TABLE, "${TAG}_request() : return cause request params is empty.")
            return
        }
        MiddlewareProxy.addRequestToBuffer(
                MiddlewareProxy.getCurrentPageId(),
                ProtocalDef.PAGEID_GZ,
                QueueManagement.getInstanceId(this),
                buildRequestParams())
        if (flush) {
            MiddlewareProxy.requestFlush(false)
        }
    }

    fun stopClient() {
        MiddlewareProxy.unSubscribe(this)
    }

    override fun receive(struct: StuffBaseStruct?) {
        if (struct is StuffTableStruct) {
            // 解析表格数据
            val dataModel = mParseIntercept?.parseData(struct, mIds) ?: parseData(struct, mIds)
            if (dataModel == null || dataModel.size <= 0) {
                Log.w(Log.AM_HQ_TABLE, "${TAG}_receive() : parsed data is empty.")
            }
            GlobalHandler.post { mCallback?.callback(dataModel, mIds) }
        }
    }

    private fun parseData(struct: StuffBaseStruct, dataIds: IntArray): DPDataModel? {
        if (struct is StuffTableStruct) {
            val fileStruct = struct
            val idsCount = dataIds.size
            val rows = fileStruct.row
            val dpDataModel = DPDataModel()
            for (i in 0..idsCount - 1) {
                val dataID = dataIds[i]
                val colValues = fileStruct.getData(dataID)
                val colColors = fileStruct.getDataColor(dataID)
                if (colValues == null || colValues.size != rows || colColors == null || colColors.size != rows) {
                    continue
                }
                dpDataModel.putValues(dataID, colValues)
                dpDataModel.putColors(dataID, colColors)
            }
            if (dpDataModel.size == idsCount) {
                return dpDataModel
            }
            return null
        }
        return null
    }

    open protected fun buildRequestParams() : String {
        val list = reqStocks
        if (list.isNotEmpty()) {
            val size = list.size
            var stockInfo: EQBasicStockInfo?
            val stocks = StringBuilder()
            val markets = StringBuilder()
            for (i in 0..size - 1) {
                stockInfo = list[i]
                stocks.append(stockInfo.mStockCode).append(EQConstants.REQUEST_SELFSTOCKLIST_SEPARATOR)
                markets.append(stockInfo.mMarket).append(EQConstants.REQUEST_SELFSTOCKLIST_SEPARATOR)
            }
            if (stocks.isNotEmpty() && markets.isNotEmpty()) {
                return String.format("stocklist=%s\nmarketlist=%s\nrowcount=%s", stocks.toString(), markets.toString(), size.toString())
            }
        }
        return EQConstants.INVALID_STR_VALUE
    }

    inner class DPDataModel {

        private val valuesMap: SparseArray<Array<String>> = SparseArray<Array<String>>()
        private val colorsMap: SparseArray<IntArray> = SparseArray<IntArray>()

        fun putValues(id: Int, values: Array<String>) {
            valuesMap.put(id, values)
        }

        fun getvalues(id: Int): Array<String>? {
            return valuesMap[id]
        }

        fun putColors(id: Int, colors: IntArray) {
            colorsMap.put(id, colors)
        }

        fun getColors(id: Int): IntArray? {
            return colorsMap[id]
        }

        val size: Int
            get() = valuesMap.size()
    }

    interface DataCallback {
        /**
         * called on UI thread.
         */
        fun callback(data : DPDataModel?, ids : IntArray)
    }

    interface DataParserInterception {
        fun parseData(struct: StuffBaseStruct, dataIds: IntArray) : DPDataModel?
    }
}