package com.hexin.android.component.hq.kcb

import android.content.Context
import android.text.TextUtils
import android.util.AttributeSet
import android.view.MotionEvent
import android.view.View
import android.widget.LinearLayout
import com.hexin.android.component.common.BaseLinearComponent
import com.hexin.android.component.curve.data.CurveDataBean
import com.hexin.android.component.curve.view.ActionEvent
import com.hexin.android.component.curve.view.CurveUnit
import com.hexin.android.component.curve.view.CurveView
import com.hexin.android.component.curve.view.FenshiUnit
import com.hexin.android.component.dpkj.DPKJFenshiOverLayPage
import com.hexin.android.component.hangqing.hushen.StockSortListComponent
import com.hexin.android.component.hangqing.hushen.StockSortListPresenter
import com.hexin.android.component.hq.FenshiValueBar
import com.hexin.app.event.action.EQGotoUnknownFrameAction
import com.hexin.app.event.param.EQGotoParam
import com.hexin.app.event.param.EQParam
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.middleware.MiddlewareProxy
import com.hexin.middleware.ProtocalDef
import com.hexin.plat.android.R
import com.hexin.util.config.EQConstants

/**
 * 科创板页面
 * Created by yuegang on 19-5-22.
 */
class KCBPage(context: Context?, attrs: AttributeSet?) : BaseLinearComponent(context, attrs), CurveView.OnCurveViewClickListener , FenshiUnit.IFenshiDataListener {


    private val indexFenShiArea by lazy { findViewById<LinearLayout>(R.id.ll_kcb_index_area) }
    private val fenshiValueBar by lazy { findViewById<FenshiValueBar>(R.id.kcb_fenshi_value_bar) }
    private val fenshiLayout by lazy { findViewById<DPKJFenshiOverLayPage>(R.id.curveview) }
    private val sortListComponent by lazy { findViewById<StockSortListComponent>(R.id.kcb_stock_list_layout) }

    private var lastMsg: String? = EQConstants.INVALID_STR_NULL

    override fun onFinishInflate() {
        super.onFinishInflate()
        fenshiLayout.curveRootView?.let {
            for(item in it.childs) {
                if (item is FenshiUnit) {
                    item.curveGraph?.setOnCurveViewClickListener(this)
                } else if (item is CurveUnit) {
                    item.curveGraph?.setOnCurveViewClickListener(this)
                }
            }
        }
        fenshiLayout.fenshiUnit?.setDataListener(this)
        sortListComponent.setPageType(StockSortListPresenter.SelectModelType.TYPE_KCB)
    }

    override fun onCurveViewClicked(view: CurveView?, actionId: Int, event: MotionEvent?, extra: Any?) {
        if (event?.action == MotionEvent.ACTION_UP &&
                (actionId == ActionEvent.ACTION_DP_OVERLAY_CLICKED || actionId == ActionEvent.ACTION_CURVE_TECH_CLICKED)) {
            val stockInfo = getStockInfo()
            if (stockInfo != null && !TextUtils.isEmpty(stockInfo.mStockCode)) {
                var market = stockInfo.mMarket
                if (TextUtils.isEmpty(market)) {
                    market = MiddlewareProxy.getStockMarket(stockInfo.mStockCode)
                    stockInfo.mMarket = market
                }
                if (TextUtils.isEmpty(market))
                    return
                val gotoFrameAction = EQGotoUnknownFrameAction.createEQAction(ProtocalDef.FRAMEID_STOCK_DAPAN_PRICE, market)
                val basicStockInfo = EQBasicStockInfo(fenshiValueBar.mNameFromServer, stockInfo.mStockCode, market)
                val gotoParam = EQGotoParam(EQParam.PARAM_TYPE_STOCKINFO, basicStockInfo)

                gotoFrameAction.param = gotoParam
                gotoFrameAction.isAddToStackDirect = true
                MiddlewareProxy.executorAction(gotoFrameAction)
            }
        }
    }

    private fun getStockInfo() : EQBasicStockInfo? {
        return fenshiLayout.curveRootView?.let {
            for(item in it.childs) {
                if (item is FenshiUnit) {
                    return item.stockInfo
                }
            }
            null
        }
    }

    override fun onForeground() {
        fenshiValueBar.onForeground()
        if (KCBZStockInfoManager.getInstance().kcbzStockInfo == null) { //若拿到的科创办指为空，则隐藏指数区域
            fenshiLayout.onBackground()
            setIndexFenShiAreaVisibility(View.GONE)
        } else {
            if (indexFenShiArea.visibility != VISIBLE) {
                setIndexFenShiAreaVisibility(View.VISIBLE)
            }
            fenshiLayout.stockInfo = KCBZStockInfoManager.getInstance().kcbzStockInfo
            fenshiLayout.setBgColorRes(R.color.white_FFFFFF)
            fenshiLayout.onForeground()
        }
        sortListComponent.onForeground()
    }

    override fun initTheme() {
        sortListComponent.initTheme()
        fenshiValueBar.initTheme()
        if (KCBZStockInfoManager.getInstance().kcbzStockInfo != null) {  //若拿到的科创板指不为空，则刷新指数区域
            fenshiLayout.invalidate()
        }
    }

    override fun onBackground() {
        fenshiValueBar.onBackground()
        fenshiLayout.onBackground()
        sortListComponent.onBackground()
    }

    override fun onReceiveFenshiData(curveDataBean: CurveDataBean) {
        val msg = curveDataBean.getmMsg()?.mResponseMsg
        if (lastMsg != msg && !curveDataBean.isCacheData) {
            if (msg != null) {
                setIndexFenShiAreaVisibility(View.GONE)
            } else {
                setIndexFenShiAreaVisibility(View.VISIBLE)
            }
            lastMsg = msg
        }
    }

    private fun setIndexFenShiAreaVisibility(visibility: Int) {
        if (indexFenShiArea.visibility != visibility) {
            indexFenShiArea.visibility = visibility
        }
    }


}