package com.hexin.android.component.hq.time

import android.content.Context
import android.graphics.Color
import android.util.AttributeSet
import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import com.hexin.android.component.common.BaseRelativeComponent
import com.hexin.android.component.fenshitab.data.DxjlDataModel
import com.hexin.android.component.hq.DXJLRequest
import com.hexin.android.component.hq.DXJLRequest.DATA_TYPE_A
import com.hexin.android.component.hq.JumpHelper
import com.hexin.android.component.hq.item.HSItemViewConfig
import com.hexin.android.component.hq.item.HSItemViewConfig.TYPE_BID_VIEW
import com.hexin.android.view.ShadowDrawable
import com.hexin.app.event.EQEvent
import com.hexin.app.event.action.EQGotoFrameAction
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.middleware.MiddlewareProxy
import com.hexin.middleware.ProtocalDef
import com.hexin.middleware.QueueManagement
import com.hexin.plat.android.R
import com.hexin.android.theme.ThemeManager
import com.hexin.util.HexinUtils
import com.hexin.util.Log
import io.reactivex.android.schedulers.AndroidSchedulers

class BidStock(context: Context, attrs: AttributeSet?) : BaseRelativeComponent(context, attrs), View.OnClickListener {

    private val itemIds = arrayOf(R.id.bid_stock_item_1, R.id.bid_stock_item_2,
        R.id.bid_stock_item_3, R.id.bid_stock_item_4)
    private val splitIds = arrayOf(R.id.bid_stock_split_1, R.id.bid_stock_split_2,
        R.id.bid_stock_split_3)
    private val mBidItems by lazy { Array(itemIds.size) {i -> BidItem(findViewById(itemIds[i]), i)} }
    private val mSplits by lazy { Array(splitIds.size) {i -> findViewById<View>(splitIds[i])} }
    private val mMoreTv by lazy { findViewById<TextView>(R.id.hushen_firstpage_more_text) }
    private val mBidStockContainer by lazy { findViewById<LinearLayout>(R.id.bid_stock_list_container) }
    private val mTipsTv by lazy { findViewById<TextView>(R.id.bid_stock_tips) }
    private val mShadowDrawable by lazy { ShadowDrawable(resources.getDimensionPixelOffset(R.dimen.dp_4), resources.getDimensionPixelOffset(R.dimen.dp_4)) }
    private val mClient = DXJLRequest()
    private var mIsFirstRequest = true
    private val mDXJLModels = ArrayList<DxjlDataModel>(itemIds.size)

    override fun onFinishInflate() {
        super.onFinishInflate()
        findViewById<View>(R.id.bid_stock_more_layout).setOnClickListener(this)
        mBidStockContainer.background = mShadowDrawable
        mClient.setDataType(DATA_TYPE_A)
        mClient.setMaxNum(mBidItems.size)
    }

    override fun onClick(v: View?) {
        v?.let {
            if (it.id == R.id.bid_stock_more_layout) {
                goToBidStockPage()
            } else {
                val bidItem = it.tag
                if (bidItem is BidItem) {
                    goToFenshi(bidItem)
                }
            }
        }
    }

    override fun onForeground() {
        initTheme()
        requestData()
    }

    private fun requestData() {
        val publishSubject = mClient.sendRequest(false)
        if (mIsFirstRequest) {
            mIsFirstRequest = false
            val disposable = publishSubject.observeOn(AndroidSchedulers.mainThread())
                .subscribe({model ->
                    val models = model?.dataModels
                    if (models != null && !models.isEmpty()) {
                        updateData(models)
                    } else {
                        if (mDXJLModels.isEmpty()) {
                            showOrHideTips(true)
                        }
                        Log.w(Log.AM_HQ_TABLE, "BidStock_reqDxjlData_callback(): data is empty.")
                    }
                }, {throwable -> Log.printStackTrace(throwable)})
        }
    }

    /**
     * @param models 需保证是有效的，即size > 0
     * */
    private fun updateData(models: MutableList<DxjlDataModel>) {
        showOrHideTips(false)
        mDXJLModels.addAll(0, models)
        for (i in mDXJLModels.size - 1 downTo itemIds.size) {
            mDXJLModels.removeAt(i)
        }
        for ((index, bidItems) in mBidItems.withIndex()) {
            bidItems.setData(if (index < mDXJLModels.size) mDXJLModels[index] else null)
        }
        for (i in mDXJLModels.size - 1 until mBidItems.size - 1) {
            mSplits[i].visibility = View.INVISIBLE
        }
    }

    private fun showOrHideTips(isShow: Boolean) {
        mTipsTv.visibility = if (isShow) View.VISIBLE else View.GONE
        val splitVisible = if (isShow) View.INVISIBLE else View.VISIBLE
        for (split in mSplits) {
            split.visibility = splitVisible
        }
    }

    override fun onBackground() {
        super.onBackground()
        mClient.sendRequest(true)
        QueueManagement.remove(mClient)
    }

    override fun initTheme() {
        mMoreTv.setTextColor(ThemeManager.getColor(context, R.color.gray_999999))
        mTipsTv.setTextColor(ThemeManager.getColor(context, R.color.gray_323232))
        mShadowDrawable.setColor(ThemeManager.getColor(context, R.color.shadow))
        for (bidItem in mBidItems) {
            bidItem.initTheme()
        }
        for (split in mSplits) {
            split.setBackgroundColor(ThemeManager.getColor(context, R.color.gray_EEEEEE))
        }
    }

    private fun goToFenshi(bidItem: BidItem) {
        if (bidItem.mStockInfo == null) {
            return
        }
        JumpHelper.gotoPage(bidItem.mStockInfo,
            "yidong.${HSItemViewConfig.getPosition(TYPE_BID_VIEW)}.ggyd.${bidItem.position + 1}", true)
    }

    private fun goToBidStockPage() {
        val action = EQGotoFrameAction(EQEvent.EVENT_TYPE_CLICK, ProtocalDef.FRAMEID_DXJL_MAIN)
        MiddlewareProxy.executorAction(action)
    }

    inner class BidItem(container: LinearLayout, val position: Int) {
        private val mTimeTv = container.findViewById<TextView>(R.id.bid_stock_item_time)
        private val mStockNameTv = container.findViewById<TextView>(R.id.bid_stock_item_stock_name)
        private val mStateTv = container.findViewById<TextView>(R.id.bid_stock_item_state)
        private val mInfoTv = container.findViewById<TextView>(R.id.bid_stock_item_info)
        private var mStateColor = Color.GRAY
        var mStockInfo: EQBasicStockInfo? = null
        private set
        init {
            container.setOnClickListener(this@BidStock)
            container.tag = this
        }
        fun initTheme() {
            mTimeTv.setTextColor(ThemeManager.getColor(context, R.color.gray_999999))
            mStockNameTv.setTextColor(ThemeManager.getColor(context, R.color.gray_323232))
            mStateTv.setTextColor(HexinUtils.getTransformedColor(mStateColor, context))
            mInfoTv.setTextColor(HexinUtils.getTransformedColor(mStateColor, context))
        }

        fun setData(model: DxjlDataModel?) {
            mTimeTv.text = model?.formatTime
            mStockNameTv.text = model?.stockName
            mStateTv.text = model?.showTypeName
            mInfoTv.text = model?.value
            mStockInfo = if (model != null) {
                mStateColor = model.dataColor
                mStateTv.setTextColor(HexinUtils.getTransformedColor(mStateColor, context))
                mInfoTv.setTextColor(HexinUtils.getTransformedColor(mStateColor, context))
                EQBasicStockInfo(model.stockName, model.stockCode, model.marketId)
            } else {
                null
            }
        }
    }
}
