package com.hexin.android.component.yidong.dpbidyd

import android.content.Context
import android.text.TextUtils
import android.util.AttributeSet
import android.util.SparseArray
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import com.hexin.android.component.caitong.firstpage.HangqingRefreshManager
import com.hexin.android.component.hangqing.dingpan.DingPanDataBean
import com.hexin.android.component.hangqing.dingpan.DingPanDataManager
import com.hexin.android.component.hangqing.dingpan.OnDingPanRrefreshListener
import com.hexin.android.theme.ThemeManager
import com.hexin.android.ui.Component
import com.hexin.app.event.param.EQParam
import com.hexin.app.event.struct.EQBasicStockInfo
import com.hexin.plat.android.R
import com.hexin.util.DateUtil
import kotlinx.android.synthetic.main.layout_dp_bid_yd_item.view.*

/**
 * Description: TODO
 * Author: 刘颖杰
 * Date: 10/15/20
 * Email: liuyingjie2@myhexin.com
 */
class DpbidYdTopItem @JvmOverloads constructor(
        context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : ConstraintLayout(context, attrs, defStyleAttr), OnDingPanRrefreshListener, Component, HangqingRefreshManager.OnHangqingRefreshListener {

    var isOnForground = false
    var stockInfo: EQBasicStockInfo? = null

    override fun onFinishInflate() {
        super.onFinishInflate()
        tv_time.setTextColor(resources.getColor(R.color.gray_323232))
        tv_content.setTextColor(resources.getColor(R.color.gray_323232))
    }

    public fun setTime(time: String) {
        tv_time.text = time
    }

    public fun setContent(content: String) {
        tv_content.text = content
    }

    override fun onDingPanRefresh(dataBean: DingPanDataBean?) {
        if (dataBean == null) return
        var time = dataBean.abnormal?.time;
        if (time != null) {
            time = DateUtil.transformDateFormat(time, DateUtil.PATTERN_HH_mm_ss, DateUtil.PATTERN_HHmm)
        }
        tv_time.text = time
        tv_content.text = dataBean.abnormal?.title
        val stockInfos = dataBean.abnormal?.stockInfos
        if (stockInfos == null || stockInfos.isEmpty()) return
        val firstItem = stockInfos[0]
        if (!TextUtils.isEmpty(firstItem.code)) {
            val stockInfoList = mutableListOf<EQBasicStockInfo>()
            stockInfo = EQBasicStockInfo(firstItem.name, firstItem.code)
            stockInfoList.add(stockInfo!!)
            requestHQ(stockInfoList)
        }
    }

    fun requestHQ(eqBasicStockInfoList: List<EQBasicStockInfo>) {
        if (!isOnForground) return
        HangqingRefreshManager.getInstance().addOnHangqingRefreshListener(this, eqBasicStockInfoList)
    }

    override fun lock() {
    }

    override fun onPageFinishInflate() {
    }

    override fun onBackground() {
        isOnForground = false
        val dingPanDataManager: DingPanDataManager = DingPanDataManager.getInstance()
        dingPanDataManager.removeOnDingPanRefreshListener(this)
        HangqingRefreshManager.getInstance().removeOnHangqingRefreshListener(this)
    }

    override fun onRemove() {
    }

    override fun unlock() {
    }

    override fun parseRuntimeParam(param: EQParam?) {
    }

    override fun onForeground() {
        isOnForground = true
        val dingPanDataManager: DingPanDataManager = DingPanDataManager.getInstance()
        dingPanDataManager.changeRefreshFrequency(DingPanDataManager.TIME_REFRESH_FREQUENCY_DINGPAN)
        dingPanDataManager.addOnDingPanRefreshListener(this)
    }

    override fun OnNotifyProcess(url: String?): Int {
        return 0
    }

    override fun onActivity() {
    }

    override fun onHangqingRefresh(infoMap: MutableMap<String, SparseArray<String>>?) {
        val stockInfoTemp = stockInfo
        if (infoMap == null || infoMap.isEmpty() || stockInfoTemp == null) {
            return
        }
        val key = HangqingRefreshManager.combinationStockCodeAndMarket(stockInfoTemp.mStockCode, stockInfoTemp.mMarket)
        val dataInfoSparseArray = infoMap[key]
        dataInfoSparseArray?.let {
            val stockName = it[55]
            val stockPriceUpsAndDowns = it[34818]
            setStockPriceUpsAndDownsColor(tv_value, stockName, stockPriceUpsAndDowns)
        }
    }

    private fun setStockPriceUpsAndDownsColor(tvValue: TextView, stockName: String, stockPriceUpsAndDowns: String) {
        var valueStr = ""
        var color = ThemeManager.getColor(context, R.color.text_dark_color)
        var backgroundDrawable = resources.getDrawable(R.drawable.transparent)
        if (!TextUtils.isEmpty(stockPriceUpsAndDowns)) {
            if (stockPriceUpsAndDowns.contains("%")) {
                try {
                    valueStr = stockPriceUpsAndDowns.substring(0, stockPriceUpsAndDowns.indexOf("%"))
                    val value = valueStr.toDouble()
                    if (value < 0) {
                        color = ThemeManager.getColor(context, R.color.dpyd_top_item_price_down_text)
                        backgroundDrawable = resources.getDrawable(ThemeManager.getDrawableRes(context, R.drawable.shape_dpyd_item_down_value))
                    } else if (value > 0) {
                        color = ThemeManager.getColor(context, R.color.dpyd_top_item_price_up_text)
                        backgroundDrawable = resources.getDrawable(ThemeManager.getDrawableRes(context, R.drawable.shape_dpyd_item_up_value))
                        if (!valueStr.contains("+")) {
                            valueStr = "+$valueStr"
                        }
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        }


        if (!TextUtils.isEmpty(valueStr)) {
            tvValue.text = "$stockName  $valueStr%"
        } else {
            tvValue.text = ""
        }
        tvValue.setTextColor(color)
        tv_value.setBackgroundDrawable(backgroundDrawable)
    }

}