package com.hexin.android.component.yidong.dpbidyd

import android.content.Context
import android.graphics.*
import android.os.Build
import android.util.AttributeSet
import android.view.View
import com.hexin.android.component.firstpage.feedflow.pmzj.histogram.HistogramViewHelper
import com.hexin.plat.android.R
import com.hexin.util.Log
import com.hexin.util.StringUtils

/**
 * 折线图支持：
 *
 * 1.全部值必须为正值
 * 2.左右两侧双Y轴
 * 3.折线阴影
 *
 * 目前支持的线条类型比较少,所以draw的具体实现在该类中,后续如果有拓展,请把draw的工作通过
 * 传Canvas、绘制区域、转换系数 交个不同的LineModel去做
 *
 * Create by likai@myhexin.com on 2018-11-28
 */

class LineChartView(context: Context?, attrs: AttributeSet?) : View(context, attrs) {

    companion object {
        const val TAG = "LineChartView"
        const val X_TEXT_SPACE = 16 // X坐标文字与线的距离
        const val Y_TEXT_SPACE = 8 // Y坐标文字与线的距离
        const val MIN_LIMIT_Y = 10 // 数据里的最小值小于这个数时,Y坐标的最小值将为0
    }

    private var mConfig = LineChartAxisConfig(LineChartAxisConfig.MIN_VALUE_COUNT,
            LineChartAxisConfig.MIN_HOR_LINE_COUNT, LineChartAxisConfig.MIN_VER_LINE_COUNT) // 画坐标轴用到的配置参数
    // 下面的参数都是存的宽高位置值
    private val mDrawArea = RectF(0f, 0f, 0f, 0f) // 能用来绘制的区域
    private lateinit var mXTextPoint: Array<Float> // 所有X轴下方文本绘制的锚点位置
    private lateinit var mYPoint: Array<Float> // 画水平横线的Y坐标,同时也作为画Y方向坐标值的锚点
    // 绘制用的画笔
    private lateinit var mPaint: Paint
    private lateinit var mLinePaint: Paint // 画线专用的画笔

    private val mLinesOnLeftAxis = arrayListOf<LineModel>() // 以左侧Y轴为基准画的折线
    private val mLinesOnRightAxis = arrayListOf<LineModel>() // 以右侧Y轴为基准画的折线
    // 下面参数都是存放的数值,表示实际的值
    private var mMinLeftY = 0F // 左侧Y坐标与X轴相交原点(在View的左下方)的数值
    private var mGapLeftY = 0F // 左侧每一根线之间的Y坐标差值,大于0时才能画
    private var mMinRightY = 0F // 右侧Y坐标与X轴相交原点(在View的右下方)的数值
    private var mGapRightY = 0F// 右侧每一根线之间的Y坐标差值,大于0时才能画
    // 下面的系数用来把一个实际数值转换成在View的Canvas中对应的位置
    private var mLeftYCoeVToH = 0f // 数值在左侧Y坐标轴里表示的高度系数 数值 * mLeftYCoeVToH = 左侧Y轴坐标
    private var mRightYCoeVToH = 0f // 数值在右侧Y坐标轴里表示的高度系数 数值 * mRightYCoeVToH = 右侧Y轴坐标
    private var mXStep = 0f // 折线的点与点之间的X方向间距

    private var mWaterFlagBmp : Bitmap? = null //水印

    override fun onFinishInflate() {
        super.onFinishInflate()
        // android4.3改为软件加速,防止出现绘制问题
        if (Build.VERSION_CODES.JELLY_BEAN_MR2 == Build.VERSION.SDK_INT) {
            setLayerType(LAYER_TYPE_SOFTWARE, null)
        }
        mPaint = Paint()
        mPaint.isAntiAlias = true
        mPaint.style = Paint.Style.FILL

        mLinePaint = Paint()
        mLinePaint.isAntiAlias = true

//        if(mConfig.waterMark) {
//            mWaterFlagBmp = BitmapFactory.decodeResource(resources, R.drawable.dp_bid_waterflag)
//        }
    }

    fun configAxis(config: LineChartAxisConfig) {
        mConfig = config
    }

    fun updateView() {
        postInvalidate()
    }

    /**
     * 添加以左侧Y轴为参考的曲线,如果给null不生效,想清空折线可传
     * new ArrayList<LineModel>()
     *
     * 添加完成之后需要自行调用 updateView() 更新取消图
     */
    fun addLeftLines(models: List<LineModel>?) {
        models?.let {
            addLines(true, it)
        }
    }

    /**
     * 添加以右侧Y轴为参考的曲线,如果给null不生效,想清空折线可传
     * new ArrayList<LineModel>()
     *
     * 添加完成之后需要自行调用 updateView() 更新取消图
     */
    fun addRightLines(models: List<LineModel>?) {
        models?.let {
            addLines(false, it)
        }
    }

    private fun addLines(isLeft: Boolean, models: List<LineModel>) {
        if (models.isEmpty()) {
            if (isLeft) {
                mMinLeftY = 0F
                mGapLeftY = 0F
                mLinesOnLeftAxis.clear()
            } else {
                mMinRightY = 0F
                mGapRightY = 0F
                mLinesOnRightAxis.clear()
            }
        } else {
            val limitArray = arrayListOf<Float>()
            // 这一个循环找到所有曲线里的最大和最小值
            for (model in models) {
                model.max = model.values.max()?:-1f
                model.min = model.values.min()?:-1f
                limitArray.add(model.max)
                limitArray.add(model.min)
            }
            val max = limitArray.max()?:-1f
            val min = limitArray.min()?:-1f
            if (max < min || min < 0) {
                return
            }
            val minY : Float
            val gapY : Float
            //区分对称和非对称处理
            if (mConfig.ySymmetry && mConfig.lineCount == 1) { //y轴对称
                try {
                    val firstValue = models[0].values[0]
                    val maxAbs = Math.max(Math.abs(firstValue - min), Math.abs(firstValue - max))
                    if (models.size == 1 && models[0].percentScaleType) { //百分比类型的刻度
                        val tmpGapY = if (mConfig.horLineCount <= 1) {
                            (maxAbs * 100) / firstValue
                        } else {
                            (maxAbs * 100) / (mConfig.horLineCount / 2) / firstValue
                        }
                        //保留两位小数
                        gapY = StringUtils.format2Decimal(tmpGapY.toString()).toFloat()
                        minY = -gapY
                    } else {
                        val tempMinY = Math.max(firstValue - maxAbs, 0F)
                        minY = StringUtils.format2Decimal(tempMinY).let {
                            if (StringUtils.isNumerical(it)) {
                                it.toFloat()
                            } else {
                                0F
                            }
                        }
                        gapY = StringUtils.format2Decimal(maxAbs).let {
                            if (StringUtils.isNumerical(it)) {
                                it.toFloat()
                            } else {
                                0F
                            }
                        }
                    }
                } catch (e: NumberFormatException) {
                    Log.printStackTrace(e)
                    return
                }
            } else {
                // 下面开始对Y轴的下限进行规划
                minY = if (min <= MIN_LIMIT_Y) 0F  // 最小值小于等于10时,Y轴的最下限为0
                else (min / MIN_LIMIT_Y) * MIN_LIMIT_Y // 否则我们向下进行对10取倍数
                /*
                    1. 因为 min >= minY,所以max - minY肯定大于等于models中所有折线的极差,保证绘制区域够用
                    2. (max - minY) / mConfig.horLineCount 这里算出了每根线之间的理论均值 +1再取整是为了向上取整
                       ,这样再次保证最大的值不会画出Canvas之外
                 */
                gapY = (max - minY) / mConfig.horLineCount + 1
            }
            if (isLeft) {
                mMinLeftY = minY
                mGapLeftY = gapY
                mLinesOnLeftAxis.clear()
                mLinesOnLeftAxis.addAll(models)
            } else {
                mMinRightY = minY
                mGapRightY = gapY
                mLinesOnRightAxis.clear()
                mLinesOnRightAxis.addAll(models)
            }
        }

        calculateXYPoint()
        calculateCoefficient()
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        mDrawArea.set(paddingLeft.toFloat(), paddingTop.toFloat(),
                (measuredWidth - paddingRight).toFloat(), (measuredHeight - paddingBottom).toFloat())
        calculateXYPoint()
        calculateCoefficient()
    }

    /**
     * 计算所有X,Y基本点的位置
     */
    private fun calculateXYPoint() {
        // 先把X方向的文本位置算完
        val size = mConfig.xText?.size?:0
        if (size > 0) {
            mXTextPoint = Array(size) {0f}
            mXTextPoint[0] = mDrawArea.left // 第一个点的位置
            mXTextPoint[size - 1] = mDrawArea.right // 最后一个点的位置
            //点之间的距离
            val gapX = (mDrawArea.right - mDrawArea.left) / (size - 1)
            for (i in 1..size - 2) {
                mXTextPoint[i] = mDrawArea.left + gapX * i
            }
        } else {
            mXTextPoint = Array(0) {0f}
        }

        // 再来计算画那几根横线的Y坐标
        mYPoint = Array(mConfig.horLineCount + 1) {0f} // 这里+1是加了一根X轴
        // 最后一条线的位置
        val bottomLineY = mDrawArea.bottom - mConfig.xTextSize - X_TEXT_SPACE
        // 每道线之间的距离
        val gapY = (bottomLineY - mDrawArea.top) / mConfig.horLineCount
        // 画线是从下往上画
        mYPoint[0] = bottomLineY
        for (i in 1..mConfig.horLineCount) {
            mYPoint[i] = bottomLineY - gapY * i + 1
        }
    }

    /**
     * 计算转换系数, 每单位的数值能换算为多少高度
     */
    private fun calculateCoefficient() {
        mLeftYCoeVToH = if (mGapLeftY == 0F) 0f
        else {
            (mYPoint[0] - mYPoint[mConfig.horLineCount]) / (mGapLeftY * mConfig.horLineCount)
        }

        mRightYCoeVToH = if (mGapRightY == 0F) 0f
        else {
            (mYPoint[0] - mYPoint[mConfig.horLineCount]) / (mGapRightY * mConfig.horLineCount)
        }

        mXStep = if (mConfig.valueCount - 1 == 0) 0f
        else {
            (mDrawArea.right - mDrawArea.left - mConfig.xPadding * 2) / (mConfig.valueCount - 1)
        }
    }


    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        if (mDrawArea.right - mDrawArea.left <= 0 || mDrawArea.bottom - mDrawArea.top <= 0) {
            return // 没有绘制空间了
        }
        drawDividLine(canvas)

        drawLine(canvas, mLeftYCoeVToH, mMinLeftY, mLinesOnLeftAxis)
        drawLine(canvas, mRightYCoeVToH, mMinRightY, mLinesOnRightAxis)
        drawXText(canvas)
        drawYText(canvas)
    }

    open fun drawDividLine(canvas: Canvas?) {
        //先画方框
        mPaint.strokeWidth = mConfig.horLineWidth
        mLinePaint.color = mConfig.dividLineColor
        //横线
        val rowNum = Math.max(mConfig.horLineCount, 0)
        mLinePaint.color = mConfig.horLineColor
        val pathEffect = DashPathEffect(floatArrayOf(8f, 8f), 2f)
        mLinePaint.style = Paint.Style.STROKE
        val path = Path()
        for (i in 0..rowNum) {
            path.reset()
            if (mYPoint.size > i) {
                var y = mYPoint[i]
                if (i == 0) {
                    y += 1
                    mLinePaint.pathEffect = null
                } else if (i == rowNum) {
                    mLinePaint.pathEffect = null
                } else {
                    mLinePaint.pathEffect = pathEffect
                }
                path.moveTo(mDrawArea.left, y)
                path.lineTo(mDrawArea.right, y)
                canvas?.drawPath(path, mLinePaint)
            }
        }
        mLinePaint.pathEffect = null
        //竖线
        val colNum = Math.max(mConfig.verLineCount, 0)
        for (i in 0..colNum) {
            if (mXTextPoint.size > i) {
                canvas?.drawLine(mXTextPoint[i], mDrawArea.top, mXTextPoint[i], mYPoint[0] , mLinePaint)
            }
        }
        if (mConfig.waterMark) {
            //画水印
            mWaterFlagBmp?.let {
                canvas?.drawBitmap(it, mDrawArea.right - it.width - Y_TEXT_SPACE, mDrawArea.top + Y_TEXT_SPACE, null)
            }
        }
    }

    /**
     * 画X轴下面的文本
     */
    private fun drawXText(canvas: Canvas?) = mConfig.xText?.let{
        mPaint.color = mConfig.xTextColor
        mPaint.textSize = mConfig.xTextSize
        val metrics = mPaint.fontMetrics
        for (i in 0 until mXTextPoint.size) {
            mPaint.textAlign = when (i) {
                0 -> Paint.Align.LEFT
                mXTextPoint.size - 1 -> Paint.Align.RIGHT
                else -> Paint.Align.CENTER
            }
            canvas?.drawText(it[i], mXTextPoint[i], mYPoint[0] - metrics.ascent + X_TEXT_SPACE, mPaint)
        }
    }

    /**
     * 画Y轴的文本
     */
    private fun drawYText(canvas: Canvas?) {
        mPaint.color = mConfig.yTextColor
        mPaint.textSize = mConfig.yTextSize
        // 画左边的Y坐标
        if (mGapLeftY > 0) {
            mPaint.textAlign = Paint.Align.LEFT
            val percentScale = !mLinesOnLeftAxis.filter { it.percentScaleType }.isEmpty()
            drawOneText(canvas, true, percentScale)
        }

        // 画右边的Y坐标
        if (mGapRightY > 0) {
            mPaint.textAlign = Paint.Align.RIGHT
            val percentScale = !mLinesOnRightAxis.filter { it.percentScaleType }.isEmpty()
            drawOneText(canvas, false, percentScale)
        }
    }

    private fun drawOneText(canvas: Canvas?, isLeft: Boolean, percentScale: Boolean) {
        var yPoint: Float
        var yText: String
        // 第一根线画标注,然后每隔一根画一次
        val middIndex = (mConfig.horLineCount + 1) / 2
        for (i in 0..mConfig.horLineCount) {
            yPoint = if (i == mConfig.horLineCount) {
                mYPoint[i] - mPaint.fontMetrics.ascent
            } else if (i == middIndex) {
                mYPoint[i] - mPaint.fontMetrics.ascent - HistogramViewHelper.getFontHeight(mPaint)/2f
            } else {
                mYPoint[i] - mPaint.fontMetrics.descent - mConfig.yAxisTopPadding
            }
            val yValue = if (isLeft) {
                mMinLeftY + mGapLeftY * i
            } else {
                mMinRightY + mGapRightY * i
            }
            yText = StringUtils.format2Decimal(yValue.toString())
            if (percentScale) {
                yText = yText.plus("%")
            }
            //对称坐标需要画笔颜色设置
            if (mConfig.ySymmetry) {
                mPaint.color = if (i == middIndex) {
                    mConfig.yTextColor
                } else if (i < middIndex) {
                    mConfig.yNegColor
                } else {
                    mConfig.yPosColor
                }
            }
            canvas?.drawText(yText, if (isLeft) (mDrawArea.left + mConfig.yAxisLeftPadding) else (mDrawArea.right - mConfig.yAxisLeftPadding), yPoint, mPaint)
        }
    }

    private fun drawLine(canvas: Canvas?, coe: Float, minY: Float, list: List<LineModel>?) {
        list?.let {
            val path = Path()
            val startX = mDrawArea.left + mConfig.xPadding
            var pointCount = mConfig.valueCount

            for (model in it) {
                if (model.values.isNotEmpty() && model.drawLine) {
                    // 1.准备画笔
                    mLinePaint.strokeWidth = model.lineWidth.toFloat()
                    mLinePaint.style = Paint.Style.STROKE
                    mLinePaint.color = if (mConfig.ySymmetry && !mConfig.lineColorForce) {
                        if (model.values[0] <= model.values[model.values.size - 1]) {
                            mConfig.yPosColor
                        } else {
                            mConfig.yNegColor
                        }
                    } else {
                        model.lineColor
                    }
                    // 2.开始组建折线的Path
                    path.reset()
                    path.moveTo(startX, mYPoint[0] - (model.values[0] - minY) * coe)
                    pointCount = Math.min(model.values.size, pointCount)
                    for (i in 1 until pointCount) {
                        path.lineTo(startX + i * mXStep, mYPoint[0] - (model.values[i] - minY) * coe)
                    }
                    // 3.绘制折线
                    canvas?.drawPath(path, mLinePaint)
                    if (model.hasShadow) {
                        val shadow = Path()
                        // 4.把折线加进来
                        shadow.addPath(path)
                        // 最后一个折线点与X轴垂线的交点
                        shadow.lineTo(startX + (pointCount - 1) * mXStep, mYPoint[0])
                        // 第一个折线点与X轴垂线的交点
                        shadow.lineTo(startX, mYPoint[0])
                        // 5.空间闭合,确定绘制阴影的区域
                        shadow.close()
                        val gradient = LinearGradient(
                                0f, 0f,
                                0f, getHeight().toFloat(),
                                model.shadowStartColor, model.shadowEndColor,
                                Shader.TileMode.CLAMP)
                        mLinePaint.style = Paint.Style.FILL
                        mLinePaint.shader = gradient
                        // 6.绘制阴影
                        canvas?.drawPath(shadow, mLinePaint)
                        mLinePaint.shader = null
                    }
                }
            }
        }
    }


}