package com.hexin.android.component.yidong.dpbidyd.common

import android.content.Context
import android.text.Html
import android.text.Spanned
import android.text.TextUtils
import com.hexin.android.theme.ThemeManager
import com.hexin.plat.android.R
import com.hexin.util.StringUtils
import com.hexin.util.config.EQConstants

/**
 * color helper
 * Created by yuegang on 19-3-12.
 */
class ColorHelper private constructor() {

    companion object {
        private val HTML_FONT_FALL = "<font color='#009900'>%s</font>"
        private val HTML_FONT_FALL_CB = "<font color='#0CA3B0'>%s</font>"
        private val HTML_FONT_FALL_NIGHT = "<font color='#30A431'>%s</font>"
        private val HTML_FONT_FALL_CB_NIGHT = "<font color='#15A9B6'>%s</font>"
        private val HTML_FONT_RISE = "<font color='#E93030'>%s</font>"
        private val HTML_FONT_RISE_NIGHT = "<font color='#FD4332'>%s</font>"

        private const val FLAG_NEG = "-"
        private const val FLAG_POS = "+"
        private const val FLAG_PER = "%"

        fun getColor(context : Context, value : Any?) : Int {
            return if (value is String) {
                if (TextUtils.isEmpty(value) || TextUtils.equals(EQConstants.DEFAULT_STR, value)) {
                    getThemeColor(context, R.color.gray_323232)
                } else if (value.contains(FLAG_NEG)) {
                    getThemeColor(context, R.color.green_009900)
                } else if (value.contains(FLAG_POS)) {
                    getThemeColor(context, R.color.red_E93030_FD4332)
                } else {
                    val num : String
                    if (value.endsWith(FLAG_PER)) {
                        num = value.substringBefore(FLAG_PER)
                    } else {
                        num = value
                    }
                    if (StringUtils.isNumerical(num)) {
                        if (num.toFloat() > 0.0F) {
                            getThemeColor(context, R.color.red_E93030_FD4332)
                        } else if (num.toFloat() < 0.0F) {
                            getThemeColor(context, R.color.green_009900)
                        } else {
                            getThemeColor(context, R.color.gray_323232)
                        }
                    } else {
                        getThemeColor(context, R.color.gray_323232)
                    }
                }
            } else if (value is Float || value is Int || value is Double) {
                if (value.toString().contains(FLAG_NEG)) {
                    getThemeColor(context, R.color.green_009900)
                } else if (value.toString().toFloat() > 0.0F){
                    getThemeColor(context, R.color.red_E93030_FD4332)
                } else {
                    getThemeColor(context, R.color.gray_323232)
                }
            } else {
                getThemeColor(context, R.color.gray_323232)
            }
        }

        fun formatHtmlValue(prefix : String, value : Any?, appendPercent : Boolean = false) : Spanned {
            val htmlPrefix = if (value is String || value is Float || value is Int || value is Double) {
                    if (TextUtils.isEmpty(value.toString()) || TextUtils.equals(EQConstants.DEFAULT_STR, value.toString())) {
                        value
                    } else if (value.toString().contains(FLAG_NEG)) {
                        getHtmlPrefix(false).format(appendPercent(value, appendPercent))
                    } else {
                        getHtmlPrefix(true).format(appendPercent(value, appendPercent))
                    }
                } else {
                    EQConstants.DEFAULT_STR
                }
            return Html.fromHtml(prefix.format(htmlPrefix))
        }

        private fun appendPercent(value : Any, appendPercent: Boolean) : String {
            return if (appendPercent) {
                value.toString().plus("%")
            } else {
                value.toString()
            }
        }

        fun getHtmlPrefix(rise : Boolean) : String {
            return if (!rise) {
                if (ThemeManager.getCurrentTheme() == ThemeManager.DAY) {
                     HTML_FONT_FALL
                } else {
                     HTML_FONT_FALL_NIGHT
                }
            } else {
                if (ThemeManager.getCurrentTheme() == ThemeManager.DAY) {
                    HTML_FONT_RISE
                } else {
                    HTML_FONT_RISE_NIGHT
                }
            }
        }

        private fun getThemeColor(context : Context, resId : Int) : Int {
            return ThemeManager.getColor(context, resId)
        }
    }
}