package com.hexin.android.component.yidong.dpbidyd.request

import android.text.TextUtils
import com.hexin.android.ui.NetWorkClinet
import com.hexin.middleware.GlobalHandler
import com.hexin.middleware.MiddlewareProxy
import com.hexin.middleware.ProtocalDef
import com.hexin.middleware.QueueManagement
import com.hexin.middleware.data.StuffBaseStruct
import com.hexin.middleware.data.mobile.StuffResourceStruct
import com.hexin.util.Log
import com.hexin.util.config.EQConstants
import org.json.JSONObject

/**
 * 请求抽象类
 * Created by yuegang on 19-3-6.
 */
abstract class AbsDataClient<T> : NetWorkClinet{

    private val TAG = "AbsDataClient"
    private val REQ_TEXT_KEY_ACTION = "action="
    private val RESP_TEXT_KEY_RESULT = "result"

    var mOnDataReceiveListener : OnDataReceiveListener<T>? = null

    override fun request() {
        val instanceId = QueueManagement.getInstanceId(this)
        MiddlewareProxy.request(ProtocalDef.FRAMEID_DXJL, ProtocalDef.PAGEID_DXJL_FREE, instanceId,  requestText(false), true, true, false, ProtocalDef.REQUSET_CTRL)
    }

    fun stopRequest() {
        val instanceId = QueueManagement.getInstanceId(this)
        MiddlewareProxy.request(ProtocalDef.FRAMEID_DXJL, ProtocalDef.PAGEID_DXJL_FREE, instanceId, requestText(true),
                true, false, false, ProtocalDef.REQUSET_CTRL)
        QueueManagement.remove(this)
    }

    private fun requestText(isRequestStop : Boolean) : String {
        val sb = StringBuffer()
        sb.append("key=${reqKey()}")
        sb.append(EQConstants.REQUEST_SEPARATOR)
        if (isRequestStop) {
            sb.append(REQ_TEXT_KEY_ACTION).append(EQConstants.REQUEST_ACTION_UN_SUBSCRIBE)
        } else {
            sb.append(REQ_TEXT_KEY_ACTION).append(EQConstants.REQUEST_ACTION_SUBSCRIBE)
        }
        sb.append(EQConstants.REQUEST_SEPARATOR)
        sb.append("data_id_list=").append(5).append(EQConstants.REQUEST_SEPARATOR)
        sb.append("stock_list=").append("all").append(EQConstants.REQUEST_SEPARATOR)
        sb.append("max_msg_num=").append(reqNum())
        return sb.toString()
    }

    override fun receive(struct: StuffBaseStruct?) {
        if (struct is StuffResourceStruct) {
            Log.i(Log.AM_DP_BID, "${tag()}_receive(): receive data")
            if (struct.buffer != null && struct.type == StuffResourceStruct.JSON_CONTENT) {
                val structText = String(struct.buffer)
                var rootObj : JSONObject? = null
                try {
                    rootObj = JSONObject(structText)
                } catch(e: Exception) {
                    Log.printStackTrace(e)
                    Log.e(Log.AM_DP_BID, "${tag()}_receive(): json exception occur,and json is : $structText")
                }
                var respData : Boolean = false
                val data = rootObj?.let {
                    respData = it.has("result")
                    if (respData) { //应答数据
                        if (TextUtils.equals(rootObj!!.optString("result"), "success")) { //数据ok
                            parse(it.optString(reqKey()))
                        } else { //请求失败
                            null
                        }
                    } else { //增量推送
                        parse(it.optString(reqKey()))
                    }
                } ?: run {
                    null
                }
                GlobalHandler.post {
                    data?.let {
                        Log.i(Log.AM_DP_BID, "${tag()}_receive(): receive data successful.")
                        mOnDataReceiveListener?.onReceiveData(data, respData.not())
                    } ?: run {
                        Log.i(Log.AM_DP_BID, "${tag()}_receive(): receive empty cause parse failed.")
                        mOnDataReceiveListener?.onEmptyData()
                    }
                }
            } else {
                //no data
                GlobalHandler.post {
                    Log.i(Log.AM_DP_BID, "${tag()}_receive(): receive empty cause struct buffer is null or not a json format.")
                    mOnDataReceiveListener?.onEmptyData()
                }
            }
        } else {
            GlobalHandler.post {
                Log.i(Log.AM_DP_BID, "${tag()}_receive(): receive struct is not a StuffResourceStruct data")
                mOnDataReceiveListener?.onEmptyData()
            }
        }
    }

    protected abstract fun parse(json : String) : T?

    protected abstract fun reqKey() : String

    protected abstract fun reqNum() : Int

    open protected fun tag() : String {
        return TAG
    }

    interface OnDataReceiveListener<in T> {

        fun onReceiveData(data : T, append : Boolean)

        fun onEmptyData()
    }
}