package com.hexin.android.component.yidong.dpbidyd.request

import android.text.TextUtils
import com.hexin.android.component.firstpage.feed.toutiao.data.RealTimeStockData
import com.hexin.android.component.yidong.YidongStockInfo
import com.hexin.android.component.yidong.dpbidyd.model.BidSelectionDataBean
import com.hexin.middleware.ProtocalDef
import com.hexin.util.Log
import com.hexin.util.gson.GsonUtil
import org.json.JSONArray
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.functions.Consumer

/**
 * 大盘集合竞价选古数据请求和管理
 * Created by yuegang on 19-3-6.
 */
class StockSelectionDataClient : AbsDataClient<BidSelectionDataBean>() {

    private val TAG = "StockSelectionDataClient"
    private var REQ_KEY_SELECTION_REAL = "dpjjyd_stock"

    private var mRealTimeStockData : RealTimeStockData? = null


    override fun parse(json: String): BidSelectionDataBean? {
        return if (TextUtils.isEmpty(json)) {
            null
        } else {
            val content = if (json.trimStart().startsWith("[")) {
                val arrayObj = JSONArray(json)
                if (arrayObj.length() > 0) { //数据只有一个长度
                    arrayObj[0].toString()
                } else {
                    Log.w(Log.AM_DP_BID, "${TAG}_parse(): json array length <= 0, json text is : $json")
                    null
                }
            } else {
                json
            }
            content?.let {
                val result = GsonUtil.fromJson(it, BidSelectionDataBean::class.java)
                if (result == null) {
                    Log.w(Log.AM_DP_BID, "${TAG}_parse(): result null, the origin string is : $json")
                }
                result
            }
        }
    }

    override fun reqNum(): Int {
        return 61
    }

    override fun reqKey(): String {
        return REQ_KEY_SELECTION_REAL
    }

    override fun tag(): String {
        return TAG
    }

    fun requestStockPrice(stocks : List<YidongStockInfo>, callback: Consumer<Map<String, YidongStockInfo>>) {
        if (mRealTimeStockData == null) {
            mRealTimeStockData = RealTimeStockData(ProtocalDef.FRAMEID_DP_BID_YD)
        }
        mRealTimeStockData!!.refresh(stocks)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(callback, Consumer<Throwable> { throwable ->
                    Log.printStackTrace(throwable)
                    Log.w(Log.AM_DP_BID, "${TAG}_requstStockPrice(): exception with -> $throwable")
                })
    }

    fun stopStockPriceReq() {
        mRealTimeStockData?.stopRealDataRequest()
    }

}
