package com.hexin.android.view


import android.content.Context
import android.graphics.*
import android.view.View
import com.hexin.android.component.hangqing.HQDataConstant
import com.hexin.android.theme.ThemeManager
import com.hexin.plat.android.HexinApplication
import com.hexin.plat.android.R
import com.hexin.util.Log
import com.hexin.util.config.EQConstants

/**
 * 分时标题栏中的股票代码和标签视图
 * */
class FenshiStockInfo(context: Context) : View(context) {
    companion object {
        private const val FONT = "fonts/digital.ttf"
    }

    private val paint = Paint(Paint.ANTI_ALIAS_FLAG)
    private val defaultFace = paint.typeface
    // 代码字体大小
    private val codeSize = context.resources.getDimension(R.dimen.stock_fenshi_textsize)
    // 标签字体大小
    private val tagSize = context.resources.getDimension(R.dimen.font_18)
    // 标签高度
    private val tagBgHeight = context.resources.getDimension(R.dimen.dp_11)
    // 标签与代码之间的间距
    private val codeSpace = context.resources.getDimensionPixelOffset(R.dimen.dp_4)
    // 标签文字两边的填充宽度
    private val tagPadding = context.resources.getDimensionPixelOffset(R.dimen.dp_3)
    // 标签与标签之间的间距
    private val tagSpace = context.resources.getDimensionPixelOffset(R.dimen.dp_2)
    // 跑马灯中最后一个标签和第一个标签之间的间距
    private val marqueeSpace = tagSpace * 6
    // 跑马灯滚动速度
    private val marqueeVelo = context.resources.getDimension(R.dimen.dp_10)
    // 标签的圆角大小
    private val tagCorner = context.resources.getDimension(R.dimen.dp_1)

    private var digitalFace: Typeface? = null

    init {
        try {
            var face: Typeface? = HexinApplication.getHxApplication().digitalTypeFace
            if (face == null) {
                face = Typeface.createFromAsset(context.assets, FONT)
                HexinApplication.getHxApplication().digitalTypeFace = face
            }
            digitalFace = face!!
        } catch (e: Exception) {
            Log.e(HQDataConstant.HQTAG, "FONT_TTF CAN NOIT SUPPORT")
        }
    }

    var stockTags: List<FenshiStockInfoTag>? = null
    set(value) {
        field = value
        marqueeHelper.cancel()
        buildDrawParams()
        invalidate()
    }
    var stockCode: String = EQConstants.DEFAULT_STR
    set(value) {
        field = value
        buildDrawParams()
        invalidate()
    }

    // 股票代码的宽度
    private var codeWidth = 0f
    // 开始画的位置（当未填满时，需要居中显示）
    private var startDrawX = 0f
    // 画股票代码的位置y
    private var codeY = 0f
    // 画股票标签的位置y
    private var tagY = 0f
    // 各个标签的宽度
    private var tagsWidths = ArrayList<Int>()
    // 标签的总宽度
    private var totalTagWidth = 0f
    // 各个标签共用的rect，每次绘制时会更新left,right等
    private val tagBgRect = RectF()
    private val marqueeHelper = MarqueeHelper()
    // 绘制标签的区域rect
    private val tagsRect = Rect()


    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        paint.typeface = digitalFace
        paint.textSize = codeSize
        paint.color = Color.WHITE
        canvas?.drawText(stockCode, startDrawX, codeY, paint)
        canvas?.save()
        canvas?.clipRect(tagsRect)
        var deltaX = 0f
        if (marqueeHelper.isMarquee()) {
            deltaX = marqueeHelper.getDelta()
            canvas?.translate(deltaX, 0f)
        }
        stockTags?.let {
            // 标签背景的上下边界
            val bgTop = top + (height - tagBgHeight) / 2
            val bgBottom = bottom - (height - tagBgHeight) / 2
            // 向右平移已绘制的代码的宽度
            canvas?.translate(startDrawX + codeWidth + codeSpace, 0f)
            var drawX = 0f
            paint.typeface = defaultFace
            paint.textSize = tagSize

            // |===隐藏区===|=========显示区=========|=======隐藏区=======|
            // |==tag1== ==tag2== ==tag3== | ==tag1== ==tag2== ==tag3==|
            // |==deltaX===|=========显示区=========|=======隐藏区=======|
            for (i in 0 until tagsWidths.size) {
                // 标签的右侧在滚动区域右侧, 绘制标签, 如tag2, tag3
                if (drawX + tagsWidths[i] > -deltaX) {
                    paint.color = ThemeManager.getColor(context, it[i].colorId)
                    tagBgRect.set(drawX, bgTop, drawX + tagsWidths[i], bgBottom)
                    canvas?.drawRoundRect(tagBgRect, tagCorner, tagCorner, paint)
                    paint.color = Color.WHITE
                    canvas?.drawText(it[i].tag, drawX + tagPadding, tagY, paint)
                }
                if (drawX + totalTagWidth + marqueeSpace < tagsRect.width() - deltaX) {
                    // 标签的左侧在显示区域内，绘制标签, 如tag1
                    paint.color = ThemeManager.getColor(context, it[i].colorId)
                    tagBgRect.set(drawX + totalTagWidth + marqueeSpace, bgTop,
                        drawX + totalTagWidth + marqueeSpace + tagsWidths[i], bgBottom)
                    canvas?.drawRoundRect(tagBgRect, tagCorner, tagCorner, paint)
                    paint.color = Color.WHITE
                    canvas?.drawText(it[i].tag, drawX + totalTagWidth + marqueeSpace + tagPadding, tagY, paint)
                }
                drawX += tagsWidths[i] + tagSpace
            }
        }
        canvas?.restore()
        startAnim()
    }

    private fun startAnim() {
        if (startDrawX > 0) {
            return
        }
        if (!marqueeHelper.isInit()) {
            marqueeHelper.initMarquee(totalTagWidth + marqueeSpace, marqueeVelo)
            if (marqueeHelper.start()) {
                invalidate()
            }
        } else {
            if (marqueeHelper.isCold()) {
                postInvalidateDelayed(MarqueeHelper.COLD_TIME)
            } else {
                invalidate()
            }
        }
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        paint.typeface = digitalFace
        paint.textSize = codeSize
        codeY = (measuredHeight - paint.ascent()) / 2
        paint.typeface = defaultFace
        paint.textSize = tagSize
        tagY = (measuredHeight - paint.descent() - paint.ascent()) / 2
        buildDrawParams()
    }

    private fun buildDrawParams() {
        paint.textSize = codeSize
        codeWidth = paint.measureText(stockCode)
        tagsWidths.clear()
        val tagAllPadding = tagPadding * 2
        totalTagWidth = 0f
        var codeSpace = 0
        stockTags?.let{
            if (!it.isEmpty()) {
                codeSpace = this.codeSpace
            } else {
                return@let
            }
            paint.textSize = tagSize
            var tagWidth: Int
            for (tag in it) {
                tagWidth = tagAllPadding + paint.measureText(tag.tag).toInt()
                totalTagWidth += tagWidth + tagSpace
                tagsWidths.add(tagWidth)
            }
            totalTagWidth -= tagSpace
        }
        // measuredWidth < totalTagWidth + codeSpace + codeWidth 说明宽度不够，从0开始画，否则应画在视图的中间位置
        startDrawX = if (measuredWidth < totalTagWidth + codeSpace + codeWidth) 0f
        else (measuredWidth - totalTagWidth - codeSpace - codeWidth) / 2
        tagsRect.set((startDrawX + codeWidth + codeSpace).toInt(), top, (right - startDrawX).toInt(), bottom)
    }
}