package com.hexin.android.weituo

import android.content.Context
import android.text.TextUtils
import com.hexin.android.component.StockMarketUtils
import com.hexin.android.view.base.DialogHelper
import com.hexin.android.weituo.yyb.WeituoAccountManager
import com.hexin.plat.android.R
import com.hexin.util.HexinUtils
import com.hexin.util.Log
import java.text.DecimalFormat

/**
 * 科创板,创业板股票限价提示
 * Created by louyancheng@myhexin.com
 */

object KCBPriceLimitManger {

    const val LIMIT_INVALID = -1.00

    const val PRICE_INVALID = ""

    const val PRICE_INPUT_LIMIT = 2

    const val TAG = "KCBPriceLimitManger"

    private var limitRatio: LimitRatio //限价比例数据结构

    private var standardPrice: StandardPrices //基准价格数据结构

    val doubleFormat = DecimalFormat("0.00")

    var isWarmingAlreadyShow = false

    init {
        limitRatio = getLimit(getKcbPriceLimitRatios())
        standardPrice = StandardPrices(PRICE_INVALID, PRICE_INVALID, PRICE_INVALID, PRICE_INVALID, PRICE_INVALID)
        Log.d(TAG, "init limitRatio = $limitRatio")
    }

    private fun getKcbPriceLimitRatios(): Array<String>? {
        return WeituoAccountManager.getInstance().wtFlagManager.getmKcbLimitRatios()
    }

    private fun isStockCodeNeedCalCankaoPrice(stockcode: String?) : Boolean{
        return StockMarketUtils.isKCB(stockcode) || StockMarketUtils.isCYB(stockcode)
    }

    /**
     * 缓存买一卖一价格
     */
    fun saveOnePrice(stockcode: String?, buyOnePrice: String?, saleOnePrice: String?) {
        if (buyOnePrice == null || saleOnePrice == null || stockcode == null) {
            return
        }
        if (!isStockCodeNeedCalCankaoPrice(stockcode)) {
            Log.e(TAG, "saveOnePrice stockcode is not kcb or cyb ! stockcode = $stockcode")
            return
        }
        if (TextUtils.equals(stockcode, standardPrice.stockcode)) {
            standardPrice.buyOnePrice = buyOnePrice
            standardPrice.saleOnePrice = saleOnePrice
        } else {
            standardPrice = StandardPrices(stockcode, buyOnePrice, saleOnePrice, PRICE_INVALID, PRICE_INVALID)
        }
        Log.d(TAG, "saveOnePrice standardPrice = $standardPrice")
    }

    /**
     * 缓存最新价
     */
    fun saveNowPrice(stockcode: String?, nowRrice: String?) {
        if (nowRrice == null || stockcode == null) {
            return
        }
        if (!isStockCodeNeedCalCankaoPrice(stockcode)) {
            Log.e(TAG, "saveOnePrice stockcode is not kcb or cyb ! stockcode = $stockcode")
            return
        }
        if (TextUtils.equals(stockcode, standardPrice.stockcode)) {
            standardPrice.nowRrice = nowRrice
        } else {
            standardPrice = StandardPrices(stockcode, PRICE_INVALID, PRICE_INVALID, nowRrice, PRICE_INVALID)
        }
        Log.d(TAG, "saveNowPrice standardPrice = $standardPrice")
    }

    /**
     * 缓存昨收价
     */
    fun saveYesterdayPrice(stockcode: String?, yesterdayPrice: String?) {
        if (yesterdayPrice == null || stockcode == null) {
            return
        }
        if (!isStockCodeNeedCalCankaoPrice(stockcode)) {
            Log.e(TAG, "saveOnePrice stockcode is not kcb or cyb ! stockcode = $stockcode")
            return
        }
        if (TextUtils.equals(stockcode, standardPrice.stockcode)) {
            standardPrice.yesterdayPrice = yesterdayPrice
        } else {
            standardPrice = StandardPrices(stockcode, PRICE_INVALID, PRICE_INVALID, PRICE_INVALID, yesterdayPrice)
        }
    }

    fun getBuyStandPrice(stockcode: String?, ztPrice: String?): String? {
        Log.d(TAG, "getBuyStandPrice stockcode = $stockcode,ztPrice = $ztPrice")
        limitRatio = getLimit(getKcbPriceLimitRatios())
        if (limitRatio.topLimit == LIMIT_INVALID) {
            return ztPrice
        }
        if (TextUtils.isEmpty(stockcode)) {
            return "--"
        }
        Log.d(TAG, "getBuyStandPrice limitRatio = $limitRatio,standardPrice = $standardPrice")
        var topPrice: Double = LIMIT_INVALID
        if (stockcode == standardPrice.stockcode) {
            val topOrder = arrayOf(standardPrice.saleOnePrice, standardPrice.buyOnePrice, standardPrice.nowRrice, standardPrice.yesterdayPrice)
            for (i in 0 until topOrder.size) {
                topPrice = calculPrice(topOrder[i], true)
                if (topPrice != LIMIT_INVALID) {
                    break
                }
            }
        }
        if (topPrice == LIMIT_INVALID) {
            return "--"
        }
        var calztPrice = LIMIT_INVALID
        if (HexinUtils.isNumerical(ztPrice)) {
            calztPrice = ztPrice!!.toDouble()
        }
        if (calztPrice != LIMIT_INVALID && topPrice > calztPrice && calztPrice > 0) {
            topPrice = calztPrice
        }
        return doubleFormat.format(topPrice)
    }

    fun getSaleStandPrice(stockcode: String?, dtPrice: String?): String? {
        Log.d(TAG, "getSaleStandPrice stockcode = $stockcode,dtPrice = $dtPrice")
        limitRatio = getLimit(getKcbPriceLimitRatios())
        if (limitRatio.downLimit == LIMIT_INVALID) {
            return dtPrice
        }
        if (TextUtils.isEmpty(stockcode)) {
            return "--"
        }
        Log.d(TAG, "getSaleStandPrice limitRatio = $limitRatio,standardPrice = $standardPrice")
        var downPrice: Double = LIMIT_INVALID
        if (stockcode == standardPrice.stockcode) {
            val topOrder = arrayOf(standardPrice.buyOnePrice, standardPrice.saleOnePrice, standardPrice.nowRrice, standardPrice.yesterdayPrice)
            for (i in 0 until topOrder.size) {
                downPrice = calculPrice(topOrder[i], false)
                if (downPrice != LIMIT_INVALID) {
                    break
                }
            }
        }
        if (downPrice == LIMIT_INVALID) {
            return "--"
        }
        var caldtPrice = LIMIT_INVALID
        if (HexinUtils.isNumerical(dtPrice)) {
            caldtPrice = dtPrice!!.toDouble()
        }
        if (caldtPrice != LIMIT_INVALID && downPrice < caldtPrice) {
            downPrice = caldtPrice
        }
        return doubleFormat.format(downPrice)
    }

    private fun calculPrice(standPrice: String, isTop: Boolean): Double {
        var topPrice = LIMIT_INVALID
        if (HexinUtils.isNumerical(standPrice)) {
            val calPrice = standPrice.toDouble()
            val limitRatio = if (isTop) limitRatio.topLimit else limitRatio.downLimit
            topPrice = calPrice * limitRatio / 100
        }
        return topPrice
    }

    private fun getLimit(limits: Array<String>?): LimitRatio {
        limits?.let {
            val size = limits.size
            if (size == 2 && HexinUtils.isNumerical(limits[0]) && HexinUtils.isNumerical(limits[1])) {
                var downLimit = limits[0].toDouble()
                var topLimit = limits[1].toDouble()
                if (downLimit > topLimit) {
                    val temp = downLimit
                    downLimit = topLimit
                    topLimit = temp
                }
                return LimitRatio(topLimit, downLimit)
            }
        }
        return LimitRatio(LIMIT_INVALID, LIMIT_INVALID)
    }

    data class LimitRatio(val topLimit: Double, val downLimit: Double)

    data class StandardPrices(var stockcode: String, var buyOnePrice: String, var saleOnePrice: String, var nowRrice: String, var yesterdayPrice: String)

    fun showLimitDialog(context: Context){
        if(HexinUtils.isDoubleClick())
            return
        DialogHelper.showMessage(context,
                context.getString(R.string.price_limit_dialog_title),
                context.getString(R.string.price_limit_dialog_content),
                context.getString(R.string.i_got_it), null)
    }
}
