package com.hexin.android.weituo.view

import android.content.Context
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.PopupWindow
import android.widget.TextView
import com.hexin.android.theme.ThemeManager
import com.hexin.middleware.MiddlewareProxy
import com.hexin.plat.android.R

/**
 * 市价选择对话框
 */
class ShijiaSelectDialog {

    private var popupWindow : PopupWindow? = null
    var onSelectListener : OnSelectListener? = null

    public fun showDialog(context : Context, items : Array<String>, selectIndex : Int) {
        val view = createView(context, items, selectIndex)
        showBottomPopView(view)
    }

    public fun dismisDialog() {
        if (popupWindow != null && popupWindow!!.isShowing)
            popupWindow!!.dismiss()
        onSelectListener = null
    }

    private fun createView(context : Context, items : Array<String>, selectIndex : Int) : View {
        var contentView = LayoutInflater.from(context).inflate(R.layout.view_shijia_select_dialog, null)
        val dataBeanList = mutableListOf<ShijiaItemBean>()
        for ((index, value) in items.withIndex()) {
            val bean = ShijiaItemBean()
            bean.index = index
            bean.content = value
            bean.isSelected = selectIndex == index
            dataBeanList.add(bean)
        }
        val rvShijiaItems = contentView.findViewById<RecyclerView>(R.id.rvShijiaItems)
        val layoutManager = SyLinearLayoutManager(context, LinearLayoutManager.VERTICAL, false)
        layoutManager.isAutoMeasureEnabled = true
        rvShijiaItems.layoutManager = layoutManager
        rvShijiaItems.setBackgroundColor(ThemeManager.getColor(context, R.color.backgroud_white))
        val adapter = ItemAdapter()
        rvShijiaItems.adapter = adapter
        adapter.itemDataList = dataBeanList


        val vBottom = contentView.findViewById<View>(R.id.vBottom);
        val bottomViewLayoutParam = vBottom.layoutParams
        bottomViewLayoutParam.height = MiddlewareProxy.getUiManager().tabWidget.height
        vBottom.layoutParams = bottomViewLayoutParam
        vBottom.setOnClickListener {
            dismisDialog()
        }

        return contentView
    }

    private fun showBottomPopView(view: View) {
        view.findViewById<View>(R.id.head_layout).setOnClickListener {
            if (popupWindow != null) {
                popupWindow!!.dismiss()
            }
        }
        popupWindow = PopupWindow(view, android.view.ViewGroup.LayoutParams.MATCH_PARENT,
                ViewGroup.LayoutParams.MATCH_PARENT)
        popupWindow!!.isFocusable = true
        popupWindow!!.isOutsideTouchable = true
        popupWindow!!.animationStyle = R.style.ShijiaBottomDialogAnimation
        popupWindow!!.showAtLocation(view, Gravity.BOTTOM or Gravity.CENTER_HORIZONTAL,
                0, 0)

    }

    inner class ItemAdapter : RecyclerView.Adapter<ShijiaViewHolder>() {

        var itemDataList : List<ShijiaItemBean>? = null
        set(value) {
            field = value
            notifyDataSetChanged()
        }

        override fun onCreateViewHolder(p0: ViewGroup, p1: Int): ShijiaViewHolder {
            val view = LayoutInflater.from(p0?.context).inflate(R.layout.view_shijia_select_dialog_item, p0, false)
            return ShijiaViewHolder(view)
        }

        override fun getItemCount(): Int {
            if (itemDataList != null) {
                return itemDataList!!.size
            }
            return 0
        }

        override fun onBindViewHolder(p0: ShijiaViewHolder, p1: Int) {
            if (p0 == null || itemDataList == null) return
            val bean = itemDataList!![p1]
            p0.setData(bean)
        }

    }

    inner class ShijiaViewHolder(itemView : View) : RecyclerView.ViewHolder(itemView) {

        var tvContent : TextView = itemView.findViewById(R.id.tvContent)
        var ivSelect : ImageView = itemView.findViewById(R.id.ivSelect)

        fun setData(bean : ShijiaItemBean) {
            tvContent.text = bean.content
            ivSelect.visibility = if (bean.isSelected) View.VISIBLE else View.INVISIBLE
            if (bean.isSelected) {
                tvContent.setTextColor(ThemeManager.getColor(itemView.context, R.color.new_red))
            } else {
                tvContent.setTextColor(ThemeManager.getColor(itemView.context, R.color.gray_323232))
            }
            itemView.setOnClickListener {
                if (onSelectListener != null) {
                    onSelectListener!!.onSelect(bean.index)
                }
            }
        }
    }

    inner class ShijiaItemBean {
        var index = -1
        var content : String? = null
        var isSelected = false
    }

    interface OnSelectListener {
        fun onSelect(index : Int)
    }
}