
package com.hexin.uninstallfeedback;

import java.io.File;
import java.io.FileOutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Build;
import android.os.Environment;

public class UninstallFeedBackEngine {
    private static final long   START_TIME_GAP = 12*60*60*1000;//12个小时启动一次
    private static final String CACHE_PREFIX          = "/cache/uninstall/";
    private static final String FEEDBACK_FILENAME     = "feedback_info.txt";
    private static final String OBSERVEDFILEDIR       = "observedFile";
    private static final String LOCKFILE              = "lockFile";
    private static final String TAG                   = "Uninstall";
    private static final String SP_FILE_NAME          = "_uninstall.xml";
    private static final String SP_KEY_LAST_STARTTIME = "sp_key_last_starttime";
    private static boolean      ISLOADSUCCESS         = false;

    static {
        try {
            System.loadLibrary("uninstall_feedback");
            ISLOADSUCCESS = true;
        } catch (UnsatisfiedLinkError exception) {
            exception.printStackTrace();
            ISLOADSUCCESS = false;
        }
    }

    private static native int initObserverProcess(String userSerial, String filePath,String appDir,String observedFileDir,String lockFile);

    private UninstallFeedBackEngine() {};

    /**
     * @param packageName 程序包名
     * @param url 反馈地址
     * @param context activity上下文
     * @return 0 正常启动 -1启动失败
     */
    public static int startObserver(Context context) {
        if (!ISLOADSUCCESS)
            return -1;
        if (context == null) {
            return -1;
        }
        SharedPreferences spf = context.getSharedPreferences(UninstallFeedBackEngine.SP_FILE_NAME, Context.MODE_PRIVATE);
        if(spf!=null){
            long lastTime = spf.getLong(SP_KEY_LAST_STARTTIME, 0);
            if(System.currentTimeMillis()-lastTime<START_TIME_GAP){
                return 0;
            }
        }
        spf.edit().putLong(SP_KEY_LAST_STARTTIME, System.currentTimeMillis()).commit();
        
        // 记录用户sockets的文件路径
        String sdPath = getSDPath();
        String filePath;
        if (sdPath != null) {
            String fileNamePrefix = getMd5String(context, context.getPackageName());
            if(fileNamePrefix == null){
                fileNamePrefix = "";
            }
            filePath = sdPath + CACHE_PREFIX + fileNamePrefix+"_"+FEEDBACK_FILENAME;
        } else {
            return -1;
        }
        String appDir = "/data/data/"+context.getPackageName();
        String observedFileDir = appDir+File.separator+OBSERVEDFILEDIR;
        String lockFileDir = appDir+File.separator+LOCKFILE;
        int sdkVersion = Build.VERSION.SDK_INT;
        if (sdkVersion < 1) {
            return -1;
        }
        if (sdkVersion < 17)
            startObserverLowAPI16(filePath, appDir, observedFileDir, lockFileDir);
        else {
            String userSerial = getUserSerial(context);
            startObserverHighAPI17(userSerial, filePath, appDir, observedFileDir, lockFileDir);
        }
        return 1;
    }

    private static void makeDirOfFile(String filePath) {
        if (null == filePath)
            return;
        File file = new File(filePath);
        if (!file.exists() || !file.isDirectory()) {
            file.mkdirs();
        }
    }

    /**
     * @param urlString 格式为 http://**?uid=xxx&subid=xxxx&...
     * */
    public static void updateFeedbackURL(Context context, final String urlString) {
        if (!ISLOADSUCCESS)
            return;
        if (urlString == null)
            return;
        if(context == null){
            return;
        }
        
        String sdPath = getSDPath();
        String filePath;
        String fileName;
        if (sdPath != null) {
            filePath = sdPath + CACHE_PREFIX;
        } else {
            // 如果没有获取到sd卡直接返回
            return;
        }
        String fileNamePrefix = getMd5String(context, context.getPackageName());
        if(fileNamePrefix == null){
            fileNamePrefix = "";
        }
        fileName = filePath + fileNamePrefix+"_"+FEEDBACK_FILENAME;
        makeDirOfFile(filePath);
        writeTxtFile(urlString, fileName);
    }
    
    public static String getMd5String(Context context, String val){
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            md.update(val.getBytes());
            StringBuffer sb = new StringBuffer();
            byte [] digestBuffer = md.digest();
            for (byte b : digestBuffer) {
                sb.append(b);
            }
            return sb.toString();
        } catch (NoSuchAlgorithmException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
        return null;
    }

    private static void startObserverLowAPI16(String filePath,String appDir,String observedFileDir,String lockFile) {
        initObserverProcess(null, filePath, appDir, observedFileDir, lockFile);
    }

    private static void startObserverHighAPI17(String userSerial, String filePath,String appDir,String observedFileDir,String lockFile) {
        initObserverProcess(userSerial, filePath, appDir, observedFileDir, lockFile);
    }

    /**
     * URL检查
     * 
     * @param pInput 要检查的字符串
     * @return boolean 返回检查结果
     */
    private static boolean isUrl(String pInput) {
        if (pInput == null) {
            return false;
        }
        String regex = "^(https?|ftp|file)://[-a-zA-Z0-9+&@#/%?=~_|!:,.;]*[-a-zA-Z0-9+&@#/%=~_|]";
        Pattern p = Pattern.compile(regex);
        Matcher matcher = p.matcher(pInput);
        return matcher.matches();
    }

    /**
     * 获取so文件加载状态
     * 
     * @return boolean
     * */
    public static boolean isLoadLibrarySuccess() {
        return ISLOADSUCCESS;
    }

    private static String getUserSerial(Context context) {
        Object userManager = ((Context) context).getSystemService("user");
        if (userManager == null) {
            // Log.e(TAG, "userManager not exsit !!!");
            return null;
        }
        try {
            Method myUserHandleMethod = android.os.Process.class.getMethod("myUserHandle", (Class<?>[]) null);
            Object myUserHandle = myUserHandleMethod.invoke(android.os.Process.class, (Object[]) null);
            Method getSerialNumberForUser = userManager.getClass().getMethod("getSerialNumberForUser",
                    myUserHandle.getClass());
            long userSerial = (Long) getSerialNumberForUser.invoke(userManager, myUserHandle);
            return String.valueOf(userSerial);
        } catch (NoSuchMethodException e) {
            // Log.e(TAG, "", e);
        } catch (IllegalArgumentException e) {
            // Log.e(TAG, "", e);
        } catch (IllegalAccessException e) {
            // Log.e(TAG, "", e);
        } catch (InvocationTargetException e) {
            // Log.e(TAG, "", e);
        }
        return null;
    }

    private static String getSDPath() {
        File sdDir = null;
        boolean sdCardExist = Environment.getExternalStorageState().equals(android.os.Environment.MEDIA_MOUNTED);
        if (sdCardExist) {
            sdDir = Environment.getExternalStorageDirectory();
            return sdDir.toString();
        }
        return "/mnt/sdcard";
    }

    public static boolean writeTxtFile(String content, String fileName) {
        if (fileName == null || content == null)
            return false;
        boolean flag = false;
        File file = new File(fileName);
        FileOutputStream o = null;
        try {
            o = new FileOutputStream(file);
            o.write(content.getBytes());
            o.close();
            flag = true;
        } catch (Exception e) {
            // TODO: handle exception
            e.printStackTrace();
        } finally {}
        return flag;
    }
}
