package com.hexin.util

import android.content.res.Resources
import android.util.TypedValue
import com.hexin.plat.android.HexinApplication
import java.lang.Exception

/**
 * 该类不受字体大小设置而放大缩小
 * 注意：只支持dp，sp，px的单位
 */
class DimensHelper private constructor() {

    companion object {

        private var mTmpValue : TypedValue? = null

        private val mTmpValueLock = Any()

        fun transferDimens(typedValue : TypedValue?) : Float {
            if (typedValue != null) {
                val unit = (typedValue.data shr TypedValue.COMPLEX_UNIT_SHIFT) and TypedValue.COMPLEX_UNIT_MASK
                val resValue = TypedValue.complexToFloat(typedValue.data)
                return when (unit) {
                    TypedValue.COMPLEX_UNIT_DIP -> {
                        resValue * Resources.getSystem().displayMetrics.density
                    }
                    TypedValue.COMPLEX_UNIT_SP -> {
                        resValue * Resources.getSystem().displayMetrics.scaledDensity
                    }
                    else -> {
                        resValue
                    }
                }
            } else {
                return 0F
            }
        }

        fun getDimension(id : Int) : Float = transferDimens(
            obtainTypedValue(id)
        )

        fun getDimensionPixelOffset(id : Int) = getDimension(id).toInt()

        fun getDimensionPixelSize(id : Int) : Int {
            val value = getDimension(id)
            return if (value > 0) {
                value + 0.5F
            } else {
                value - 0.5F
            }.toInt()
        }

        fun getOriginalDensity() : Float {
            return Resources.getSystem().displayMetrics.density
        }

        private fun obtainTypedValue(id : Int) : TypedValue {
            var outValue : TypedValue? = null
            try {
                outValue = obtainTempTypedValue()
                Resources.getSystem().getValue(id, outValue, true)
            } catch (e : Exception) {
                HexinApplication.getHxApplication().resources.getValue(id, outValue, true)
            } finally {
                outValue?.let {
                    releaseTempTypedValue(it)
                }
            }
            return outValue!!
        }

        private fun obtainTempTypedValue(): TypedValue {
            var tmpValue: TypedValue? = null
            synchronized(mTmpValueLock) {
                if (mTmpValue != null) {
                    tmpValue = mTmpValue
                    mTmpValue = null
                }
            }
            return if (tmpValue == null) {
                TypedValue()
            } else {
                tmpValue!!
            }
        }

        private fun releaseTempTypedValue(value: TypedValue) {
            synchronized(mTmpValueLock) {
                if (mTmpValue == null) {
                    mTmpValue = value
                }
            }
        }
    }
}
